#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# MA-4xx Web Application Server Module
#
# Copyright (C) 2008-2009 Century Systems Co.,Ltd.
#

import re
import serial
import syslog
import time
import threading
import struct
import binascii
import commands
import os

from Utils import Serlock
from Utils import PPPstat

COM_PORT = '/dev/ttyKCMV200a'
CTRL_PORT = '/dev/ttyKCMV200w'
DIO_FILE = '/proc/driver/dio'
OTASW_CNT_THRESH = 3
LED_FILE = '/tmp/ledctrl'

POLLING_RI_INTERVAL = 2
POLLING_STATUS_INTERVAL = 10
POLLING_INFO_INTERVAL = 180

MSG_SUCCESS = 0
MSG_INVALID_REQUEST = -1
MSG_TIMEOUT = -2
MSG_UNEXPECT_RESPONSE = -3
MSG_UNKNOWN_ERROR = -4

RESULT_OK = [
        'OK',
        ]

RESULT_ERROR = [
        'ERROR',
        'CONNECT',
        'NO CARRIER',
        'DELAYED',
        'BUSY',
        ]

INFO_COMMANDS = {
        'register_status': {
            'command': 'AT$20?\r\n',
            'resp_re': re.compile("(\d)")
        },
        'phone_number': {
            'command': 'AT$21?\r\n',
            'resp_re': re.compile("(\w+)")
        },
        'manufacturer': {
            'command': 'AT+GMI\r\n',
            'resp_re': re.compile("\+GMI:(.+)")
        },
        'type': {
            'command': 'AT+GMM\r\n',
            'resp_re': re.compile("\+GMM:(.+)")
        },
        'production_number': {
            'command': 'AT+GSN\r\n',
            'resp_re': re.compile("\+GSN:(.+)")
        },
        'rom_write_number': {
            'command': 'AT@32\r\n',
            'resp_re': re.compile("(\w+)")
        },
        'module_version': {
            'command': 'AT+GMR\r\n',
            'resp_re': re.compile("\+GMR:(.+)")
        }
    }

STATUS_COMMANDS = {
        'status': {
            'command': 'AT+CLCC\r\n',
            'resp_re': re.compile("\+CLCC:,,(\d),,")
        },
        'antenna_status': {
            'command': 'AT+CAD?\r\n',
            'resp_re': re.compile("(\d)")
        },
        'antenna_level': {
            'command': 'AT$30=0\r\n',
            'resp_re': re.compile("(\d)")
        }
    }

OTASP_RESP_SEQ = [
        'OK',
        'OTASP1',
        'OTASP2',
        'OTASP3',
        'OTASP4',
        'OTASP5',
        'OTASP6',
        'OTASPOK'
    ]

OTAPA_RESP_SEQ = [
        'OK',
        'OTAPA1',
        'OTAPA2',
        'OTAPA3',
        'OTAPA4',
        'OTAPA5',
        'OTAPA6',
        'OTAPAOK'
    ]

#------------------------------------------------------------------------------
# KCMV200モニター
#------------------------------------------------------------------------------
class Kcmv200Monitor(threading.Thread):

    def __init__(self, debug=False):
        self.__ctrl_lck = Serlock.Serlock(CTRL_PORT)
        self.__com_lck = Serlock.Serlock(COM_PORT)
        self.__read_info()
        self.__read_status()

        self.__otasim = False
        if debug:
            # OTAシミュレートモード
            self.__set_ota_simulate_mode(True)

        self.__quit_event = threading.Event()
        threading.Thread.__init__(self)
        self.setDaemon(True)

    def __open_port(self, device):
        ser = None

        try:
            ser = serial.Serial()
            ser.port = device
            ser.baudrate = 115200
            ser.rtscts = 1
            ser.timeout = 1
            ser.open()
        except Exception, err:
            self.__log(syslog.LOG_ERR, "port open error: '%s'" % str(err))
            raise err

        return ser

    def __close_port(self, port):
        try:
            port.close()
        except Exception, err:
            self.__log(syslog.LOG_ERR, "port close error: '%s'" % str(err))

    def __init_port(self, port):
        # 空読み
        #self.__flush_port(port)
        self.__read_response(port)

        self.__write_command(port, 'ATE0\r\n')
        self.__read_response(port)

    def __flush_port(self, port):
        port.flushInput()
        port.flushOutput()

    def __log(self, loglevel, msg):
        log = "(KCMV200) %s" % msg
        syslog.syslog(loglevel, log)

    #--------------------------------------------------------------------------
    # コマンド送信
    #--------------------------------------------------------------------------
    def __write_command(self, port, string, loglevel=syslog.LOG_DEBUG):
        self.__log(loglevel, "send: '%s'" % string.replace('\r\n', ''))
        port.write(string)
        port.drainOutput()

    #--------------------------------------------------------------------------
    # レスポンス読み出し
    #--------------------------------------------------------------------------
    def __read_response(self, port, expect=None, retry=0,
            loglevel=syslog.LOG_DEBUG):

        result = False

        rlines = []

        while True:
            line = port.readline()
            if not line:
                if retry > 0:
                    retry -= 1
                    continue
                else:
                    break

            line = line.replace('\r', '')
            line = line.replace('\n', '')
            if not line:
                continue

            if line.startswith('AT'): # echo
                continue

            self.__log(loglevel, "recv: '%s'" % line)
            rlines.append(line)

            if expect:
                if line == expect:
                    result = True
                break

            if line in RESULT_OK:
                result = True
                break
            elif line in RESULT_ERROR:
                break

        return (result, rlines)

    #--------------------------------------------------------------------------
    # コマンド群実行
    #--------------------------------------------------------------------------
    def __exec_commands(self, commands):
        ret = {}
        try:
            port = self.__open_port(CTRL_PORT)
            self.__init_port(port)

            for key in commands.keys():
                self.__write_command(port, commands[key]['command'])
                result, resp = self.__read_response(port)
                if result:
                    matched = commands[key]['resp_re'].match(resp[0])
                    if matched:
                        ret[key] = matched.groups()[0]

            self.__close_port(port)

        except Exception, err:
            self.__log(syslog.LOG_ERR,
                    "__exec_commands error: '%s'" % str(err))

        return ret

    #--------------------------------------------------------------------------
    # OTAシミュレートモード変更
    #--------------------------------------------------------------------------
    def __set_ota_simulate_mode(self, flag):
        self.__ctrl_lck.acquire()
        self.__log(syslog.LOG_INFO,
                "set_ota_simulate_mode: flag='%s'" % str(flag))

        try:
            port = self.__open_port(CTRL_PORT)
            self.__init_port(port)

            if flag:
                val = 1
            else:
                val = 0

            self.__write_command(port, 'AT$97=%d\r\n' % val)
            result, resp = self.__read_response(port)
            self.__close_port(port)

            # ソフトウェアリセット待ち
            time.sleep(10)

            self.__otasim = flag

        except Exception, err:
            self.__log(syslog.LOG_ERR,
                    "set_ota_simulate_mode error: '%s'", str(err))

        self.__ctrl_lck.release()

    #--------------------------------------------------------------------------
    # モジュール情報読み出し
    #--------------------------------------------------------------------------
    def __read_info(self):
        self.__ctrl_lck.acquire()
        self.__log(syslog.LOG_DEBUG, "read_info: start")
        self.__info = self.__exec_commands(INFO_COMMANDS)
        self.__log(syslog.LOG_DEBUG, "read_info: '%s'" % str(self.__info))
        self.__ctrl_lck.release()

    #--------------------------------------------------------------------------
    # モジュール状態読み出し
    #--------------------------------------------------------------------------
    def __read_status(self):
        self.__ctrl_lck.acquire()
        self.__log(syslog.LOG_DEBUG, "read_status: start")
        self.__status = self.__exec_commands(STATUS_COMMANDS)
        self.__log(syslog.LOG_DEBUG, "read_status: '%s'" % str(self.__status))
        self.__ctrl_lck.release()

    #--------------------------------------------------------------------------
    # OTAシナリオ番号読み出し
    #--------------------------------------------------------------------------
    def __read_otasim_senario(self):
        fname = '/var/tmp/otasim_senario'
        senario = 0
        if os.path.exists(fname):
            try:
                f = open(fname, 'r')
                tmp = f.read()
                senario = int(tmp)
                f.close()
            except Exception, err:
                self.__log(syslog.LOG_ERR,
                        "read_otasim_senario error: '%s'" % str(err))
        return senario

    #--------------------------------------------------------------------------
    # LED設定
    #--------------------------------------------------------------------------
    def __led(self, command):
        try:
            f = open(LED_FILE, 'w+')
            f.write(command)
            f.close()
        except Exception, err:
            self.__log(syslog.LOG_ERR, "led error: '%s'" % str(err))

    #--------------------------------------------------------------------------
    # OTA実行
    #--------------------------------------------------------------------------
    def __exec_ota(self, is_otasp, senario=0):
        if senario > 0 and self.__otasim == False:
            return {'result': MSG_INVALID_REQUEST}

        self.__ctrl_lck.acquire()

        try:
            self.__led("BLINK:0 G:300,K:300\n")

            time.sleep(2)

            port = self.__open_port(CTRL_PORT)
            self.__init_port(port)

            if self.__otasim:
                if senario == 0:
                    senario = self.__read_otasim_senario()
                if senario > 0:
                    # シナリオ番号の設定
                    self.__log(syslog.LOG_INFO, "set senario #%d." % senario)
                    self.__write_command(port,
                        'AT@39=%d\r\n' % senario, syslog.LOG_INFO)
                    result, resp = self.__read_response(port, None, 0,
                        syslog.LOG_INFO)

            if is_otasp:
                self.__log(syslog.LOG_INFO, "start otasp")
                command = 'AT@30\r\n'
                resp_seq = OTASP_RESP_SEQ
            else:
                self.__log(syslog.LOG_INFO, "start otapa")
                command = 'AT@31\r\n'
                resp_seq = OTAPA_RESP_SEQ

            # OTA実行
            self.__write_command(port, command, syslog.LOG_INFO)
            for expect in resp_seq:
                result, resp = self.__read_response(port,
                        expect, 300, syslog.LOG_INFO)
                if not result:
                    if len(resp) > 0:
                        ret = {'result': MSG_UNEXPECT_RESPONSE,
                                'expect': expect, 'resp': resp[0]}
                    else:
                        ret = {'result': MSG_TIMEOUT,
                                'expect': expect}

                    self.__log(syslog.LOG_INFO,
                            "ota fail. info='%s'" % ret)
                    break
            else:
                # レスポンスを全部受信成功
                self.__log(syslog.LOG_INFO, "ota success. reset module.")
                self.__write_command(port, 'AT$40=0\r\n', syslog.LOG_INFO)
                result, resp = self.__read_response(port, None, 0,
                        syslog.LOG_INFO)
                ret = {'result': MSG_SUCCESS}

            self.__close_port(port)

        except Exception, err:
            self.__log(syslog.LOG_ERR, "exec_ota error: '%s'" % str(err))
            ret = {'result': MSG_UNKNOWN_ERROR}

        try:
            if ret['result'] == MSG_SUCCESS:
                # リセット待ち
                time.sleep(10)

                # 状態読み直し
                self.__info = self.__exec_commands(INFO_COMMANDS)
                self.__status = self.__exec_commands(STATUS_COMMANDS)

                self.__led("SET:0 G\n")
            else:
                self.__led("SET:0 R\n")

        except Exception, err:
            self.__led("SET:0 R\n")
            self.__log(syslog.LOG_ERR, "exec_ota error: '%s'" % str(err))

        self.__ctrl_lck.release()

        return ret

    #--------------------------------------------------------------------------
    # PPP発信
    #--------------------------------------------------------------------------
    def __ppp_callout(self, domain):
        stats = PPPstat.get_PPP_callout_status()

        for entry in stats:
            if entry['cipl_usable'] and entry['cipl_domain'] == domain:
                # domain名一致
                self.__log(syslog.LOG_INFO,
                    "domain '%s' ==> ppp%d." % (domain, entry['unit']))

                if not entry['cipl_sms']:
                    # sms受信機能が無効
                    self.__log(syslog.LOG_INFO,
                        "ppp%d: sms function is disabled." % entry['unit'])
                    continue

                try:
                    # 状態取得
                    status = entry['status']['value']
                    if status != 4:
                        # dormant状態でない
                        self.__log(syslog.LOG_INFO,
                            "ppp%d is not dormant(%d). discard message." % \
                                (entry['unit'], status))
                        break

                    # ACMポートを閉じる
                    self.__close_port(self.__comport)
                    self.__comport = None

                    # peerにping実行
                    peer = entry['status']['address']['peer']
                    self.__log(syslog.LOG_INFO,
                        "ppp%d: ping to peer('%s')." % \
                            (entry['unit'], peer))

                    commands.getstatusoutput("/bin/ping -w 3 %s" % peer)

                except Exception, err:
                    self.__log(syslog.LOG_INFO,
                        "ppp_callout error: '%s'" % str(err))
                break

    #--------------------------------------------------------------------------
    # SMS解析
    #--------------------------------------------------------------------------
    def __parse_sms(self, raw):
        header_fmt = "3s12s"
        header_len = struct.calcsize(header_fmt)

        if len(raw) < header_len:
            self.__log(syslog.LOG_ERR, "parse_sms: message too short.")
            return None

        try:
            tp = struct.unpack(header_fmt, raw[0:header_len])
            message_len = int(tp[0], 10)
            timestamp = tp[1]
        except:
            self.__log(syslog.LOG_ERR, "parse_sms: header error.")
            return None

        if len(raw) != message_len + 3:
            self.__log(syslog.LOG_ERR, "parse_sms: message length error.")
            return None

        userdata = raw[header_len:].split(',')
        domain_raw = userdata[0]
        l = len(domain_raw)
        if (l == 0) or (l % 2 != 0):
            self.__log(syslog.LOG_ERR, "parse_sms: illegal userdata.")
            return None

        try:
            domain = binascii.a2b_hex(domain_raw)
        except:
            self.__log(syslog.LOG_ERR,
                    "parse_sms: failed to unhexlify userdata.")
            return None

        return {'timestamp': timestamp, 'domain': domain}

    #--------------------------------------------------------------------------
    # SMS読み出し
    #--------------------------------------------------------------------------
    def __read_sms(self):

        def read_delete():
            ret = {}
            # read message
            self.__write_command(port, 'AT@02=0\r\n', syslog.LOG_INFO)
            result, resp = self.__read_response(port, None, 0,
                        syslog.LOG_INFO)
            if result:
                ret = self.__parse_sms(resp[0])

            # delete message
            self.__write_command(port, 'AT@02=1\r\n', syslog.LOG_INFO)
            result, resp = self.__read_response(port, None, 0,
                        syslog.LOG_INFO)
            return ret

        msg = {}
        self.__ctrl_lck.acquire()
        self.__log(syslog.LOG_INFO, "read_sms: start")
        try:
            port = self.__open_port(CTRL_PORT)

            self.__init_port(port)

            # read message
            self.__write_command(port, 'AT@01?\r\n', syslog.LOG_INFO)
            result, resp = self.__read_response(port, None, 0,
                        syslog.LOG_INFO)
            if result and resp[0] == '1':
                    msg = read_delete()
            else:
                # wait result code
                #self.__log(syslog.LOG_INFO, "read_sms: waiting 'SM'...")
                #result, resp = self.__read_response(port, 'SM', 10,
                #       syslog.LOG_INFO)
                #if resp and resp[0] == 'SM':
                #   self.__init_port(port)
                #   msg = read_delete()
                #else:
                #   self.__log(syslog.LOG_ERR,
                #       "read_sms error: could not read resultcode('SM').")
                self.__log(syslog.LOG_INFO, "read_sms: message not found.")

            self.__close_port(port)

            self.__log(syslog.LOG_INFO, "read_sms: msg='%s'" % msg)
            if ("domain" in msg) and (msg['domain'] != ""):
                self.__ppp_callout(msg['domain'])

        except Exception, err:
            self.__log(syslog.LOG_ERR, "read_sms error: '%s'" % str(err))

        self.__ctrl_lck.release()

    #--------------------------------------------------------------------------
    # RI状態読み出し
    #--------------------------------------------------------------------------
    def __check_RI(self):
        ret = False
        try:
            if self.__com_lck.isLocked():
                self.__log(syslog.LOG_DEBUG, "check_RI: port is locked.")
                if self.__comport:
                    self.__close_port(self.__comport)
                    self.__comport = None
                return False

            if self.__comport == None:
                self.__comport = self.__open_port(COM_PORT)

            ret = self.__comport.getRI()

        except Exception, err:
            self.__log(syslog.LOG_ERR, "check_RI error: '%s'" % str(err))

            # 開きなおし
            self.__close_port(self.__comport)
            self.__comport = self.__open_port(COM_PORT)

        self.__log(syslog.LOG_DEBUG, "check_RI: RI = %s" % str(ret))
        return ret

    #--------------------------------------------------------------------------
    # OTAスイッチ読み出し
    #--------------------------------------------------------------------------
    def __read_OTA_switch(self):
        try:
            f = open(DIO_FILE, 'r')
            tmp = f.read(2)
            f.close()
        except Exception, err:
            self.__log(syslog.LOG_ERR, "read_OTA_switch error: '%s'" % str(err))
            return False

        val = int(tmp, 16)
        if val & 0x01:
            return True
        else:
            return False

    #--------------------------------------------------------------------------
    # OTAスイッチによるOTASP実行
    #--------------------------------------------------------------------------
    def __exec_otasp_by_switch(self):
        self.__log(syslog.LOG_INFO, "exec_otasp_by_switch start")
        ret = self.__exec_ota(True, 0)
        self.__log(syslog.LOG_INFO, "exec_otasp_by_switch finish")

    #--------------------------------------------------------------------------
    # API: モジュール情報取得
    #--------------------------------------------------------------------------
    def read_info(self):
        return self.__info

    #--------------------------------------------------------------------------
    # API: モジュール状態取得
    #--------------------------------------------------------------------------
    def read_status(self):
        return self.__status

    #--------------------------------------------------------------------------
    # API: スレッド停止
    #--------------------------------------------------------------------------
    def kill_thread(self):
        self.__quit_event.set()
        self.join()

    #--------------------------------------------------------------------------
    # API: OTASP
    #--------------------------------------------------------------------------
    def otasp(self, senario=0):
        return self.__exec_ota(True, senario)

    #--------------------------------------------------------------------------
    # API: OTAPA
    #--------------------------------------------------------------------------
    def otapa(self, senario=0):
        return self.__exec_ota(False, senario)

    #--------------------------------------------------------------------------
    # モニタスレッド
    #--------------------------------------------------------------------------
    def run(self):
        self.__log(syslog.LOG_INFO, "start thread...")
        cnt = 0
        otasw_cnt = 0

        self.__comport = None

        while True:
            time.sleep(1)
            cnt += 1

            if self.__quit_event.isSet():
                break

            if self.__read_OTA_switch():
                otasw_cnt += 1
                self.__log(syslog.LOG_INFO,
                        "OTA switch is pushed (%d)." % otasw_cnt)
                if otasw_cnt >= OTASW_CNT_THRESH:
                    # OTASP実行
                    self.__exec_otasp_by_switch()
                    otasw_cnt = 0
            else:
                otasw_cnt = 0

            if (cnt % POLLING_RI_INTERVAL) == 0:
                if self.__check_RI():
                    self.__log(syslog.LOG_INFO, "detect RI.")
                    self.__read_sms()

            if (cnt % POLLING_STATUS_INTERVAL) == 0:
                self.__read_status()

            if (cnt % POLLING_INFO_INTERVAL) == 0:
                self.__read_info()

        if self.__comport:
            self.__close_port(self.__comport)

        self.__log(syslog.LOG_INFO, "finish thread")

