#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# MA-4xx/8xx Web Application Server Module
#   PPP Monitoring
# Author : Takeyoshi Kikuchi <kikuchi@centurysys.co.jp>
#
# Copyright (C) 2008 Century Systems, Inc.
# Last Modified: 2010/05/26 10:31:16 kikuchi
#

import threading
import pyiproute
from pyiproute import FILTER, AF
import syslog

from DB import config, models
moduleName = "PPPMonitor"


#===============================================================================
#
# PPP Monitor module (I/F)
#
#===============================================================================
class PPPMonitor(threading.Thread):
    def __init__(self):
        try:
            self.__ipaddr    = pyiproute.ipaddr()
            self.__iproute   = pyiproute.iproute()
            self.__ipmonitor = pyiproute.ipmonitor()
        except Exception, msg:
            print "! %s::__init__ Error '%s'" % (moduleName, str(msg))

        self.ppp_status = {}

        threading.Thread.__init__(self)
        self.setDaemon(True)

    #-------------------------------------------------------
    # Event handling
    #-------------------------------------------------------
    def handle_event(self, event_list):
        if type(event_list) != list:
            msg = "! %s::handle_event event_list is not 'list'" % \
                  moduleName
            syslog.syslog(syslog.LOG_ALERT, msg)
            return

        for entry in event_list:
            try:
                event = entry[0]
                info = entry[1]
            except Exception, msg:
                errmsg = "! %s::handle_event event entry type error, %s" % \
                         (moduleName, str(entry))
                syslog.syslog(syslog.LOG_ERR, errmsg)
                continue

            if event == 'NEWLINK' or event == 'DELLINK':
                # 'LINK' event

                if info.get('link', 'UNKNOWN') != 'ppp':
                    continue

                intf = info['name']

                if 'LOWER_UP' in info['flags']:
                    new_stat = 'UP'
                else:
                    new_stat = 'DOWN'
            elif event == 'NEWADDR' or event == 'DELADDR':
                # 'IFADDR' event

                intf = info['label']

                if intf.startswith('ppp') == False:
                    continue

                if event == 'NEWADDR' and ('peer' in info and 'local' in info):
                    new_stat = 'UP'
                else:
                    new_stat = 'DOWN'

            old_stat = self.ppp_status.get(intf, 'DOWN')

            # DOWN --> LOWER_UP
            if old_stat == 'DOWN' and new_stat == 'UP':
                self.routes_add(intf)
            elif old_stat == 'UP' and new_stat == 'DOWN':
                msg = "* %s::handle_event PPP '%s' UP->DOWN detected." % \
                      (moduleName, intf)
                syslog.syslog(syslog.LOG_INFO, msg)

            self.ppp_status[intf] = new_stat

    #-------------------------------------------------------
    # add routes
    #-------------------------------------------------------
    def routes_add(self, intf):
        msg = "* %s::routes_add PPP '%s' Down -> Up, " \
              "adding static routes..." % (moduleName, intf)
        syslog.syslog(syslog.LOG_INFO, msg)

        try:
            routes = models.Route.select_by_intf(intf)

            for entry in routes:
                self.__route_add(entry)
        except Exception, msg:
            errmsg = "! %s::routes_add failed, '%s'" % \
                     (moduleName, str(msg))
            syslog.syslog(syslog.LOG_ERR, errmsg)

    #---
    #
    def __route_add(self, route):
        try:
            newroute = {'scope'   : 'link',
                        'gateway' : {'dev' : route['gw_dev']},
                        'src'     : {},
                        'family'  : 'AF_INET',
                        'proto'   : route.get('proto', 'boot'),
                        'dst'     : {'addr'      : route['address'],
                                     'prefixlen' : route['prefixlen']},
                        'flags'   : [],
                        'table'   : route.get('table', 'main'),
                        'type'    : 'unicast'}
        except Exception, msg:
            errmsg = "! %s::__route_add Error, '%s'" % (moduleName, str(msg))
            syslog.syslog(syslog.LOG_ERR, errmsg)
            return

        self.__iproute.add(newroute)
        msg = "* %s::__route_add add new route address %s " \
              "prefixlen %d dev %s" % \
              (moduleName, route['address'], route['prefixlen'], route['gw_dev'])
        syslog.syslog(syslog.LOG_INFO, msg)

    #-------------------------------------------------------
    # initial
    #-------------------------------------------------------
    def init(self):
        devs = self.__ipaddr.get_device_list()

        for dev in devs:
            if dev.startswith('ppp'):
                self.routes_add(dev)

    #-------------------------------------------------------
    # IPmonitor open
    #-------------------------------------------------------
    def mon_init(self):
        # open ipmonitor
        mon = pyiproute.ipmonitor()

        mon.open(family=AF.INET,
                 filter=[FILTER.LINK, FILTER.IFADDR,])

        return mon

    #-------------------------------------------------------
    # EntryPoint
    #-------------------------------------------------------
    def run(self):
        msg = "* %s::run start thread..." % moduleName
        syslog.syslog(syslog.LOG_INFO, msg)

        # initialize
        self.init()

        mon = self.mon_init()

        while True:
            try:
                (nums, event_list) = mon.receive()
            except Exception, msg:
                errmsg = "! %s: mon.receive() failed, '%s'" % \
                         (moduleName, str(msg))
                syslog.syslog(syslog.LOG_ERR, errmsg)

                # re-open
                mon.close()
                mon = self.mon_init()
                continue

            if nums >= 1:
                msg = "* %s::run event received." % moduleName
                syslog.syslog(syslog.LOG_INFO, msg)

                try:
                    self.handle_event(event_list)
                except Exception, msg:
                    errmsg = "! %s:: handle_event() failed, '%s'" % \
                             (moduleName, str(msg))
                    syslog.syslog(syslog.LOG_ERR, errmsg)
            else:
                msg = "! %s:: received event count is %d" % \
                      (moduleName, nums)
                syslog.syslog(syslog.LOG_ALERT, msg)

        mon.close()
        syslog.syslog(syslog.LOG_INFO, "* PPP Monitor exit...")
        #print "*** PPP Monitor exit..."
