#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# MA-4xx/8xx Web Application Server Module
#   DHCP Server (dnsmasq) module
# Author : Takeyoshi Kikuchi <kikuchi@centurysys.co.jp>
# Copyright (C) 2007-2008 Century Systems Co.,Ltd.
#
# Last Modified: 2010/05/26 10:19:15 kikuchi
#

import copy
import cherrypy
import syslog

# Configuration DB
from DB import config, models

from Tools import jsonlib
from Tools import wsse_auth

from Utils import DaemonTools

from Handlers import BaseModule

# TemplateModule FrontEnd
from Templates.Template_dnsmasq import Template_dnsmasq

ConfigKey = "DHCPServer"
ServiceName = "dnsmasq"


#===============================================================================
#
# DHCP Server module
#
#===============================================================================
class DHCPServer(BaseModule.BaseModule):
    exposed = True

    def __init__(self, debug=False, prefix='output_test/'):
        # SuperClass (BaseModule)
        self.__super = BaseModule.BaseModule

        # モジュール内部データ管理クラス
        self.__dhcp = InternalData(self)

        # Template FrontEnd
        self.__template = Template_dnsmasq(debug, prefix)

        # Debug / Production
        self.__debug = debug

        # Superclass Constructor
        self.__super.__init__(self)

        self.log_header = "DHCPServer"

    #-----------------------------------
    # HTTP method handler (GET)
    #-----------------------------------
    def GET(self, *args, **kwargs):
        return self.__dhcp.handle_GET(*args, **kwargs)

    #-----------------------------------
    # HTTP method handler (POST)
    #-----------------------------------
    def POST(self, *args, **kwargs):
        return self.__dhcp.handle_POST(*args, **kwargs)

    #-----------------------------------
    # HTTP method handler (PUT)
    #-----------------------------------
    def PUT(self, *args, **kwargs):
        return self.__dhcp.handle_PUT(*args, **kwargs)

    #-----------------------------------
    # HTTP method handler (DELETE)
    #-----------------------------------
    def DELETE(self, *args, **kwargs):
        return "Ok"

    #---------------------------------------------
    # 設定ファイル書きだし
    #---------------------------------------------
    def write_data(self):
        # Ethernet / NTP / DHCPServer 必要
        conf = {}
        conf['NTP']        = {'ntp_info'  : models.ConfigDB.select_by_kind('NTP')}
        conf['DHCPServer'] = {'dhcp_info' : models.ConfigDB.select_by_kind(ConfigKey)}

        self.__template.write_config(conf)

        # DaemonTools 設定変更
        if self.__debug == False:
            # when production only
            self.__dhcp.update_daemon()

    #---------------------------------------------
    # extra functions
    #---------------------------------------------
    def get_data(self):
        pass


#-------------------------------------------------------------------------------
# 内部データ管理
#-------------------------------------------------------------------------------
class InternalData:
    #-----------------------------------
    #
    #-----------------------------------
    def __init__(self, module):
        self.__module = module
        self.__daemon = DaemonTools.DaemonTools()
        self.__config_init()

    #-----------------------------------
    #
    #-----------------------------------
    def __config_init(self):
        # 設定が無い場合のデフォルト
        if models.ConfigDB.select_by_kind(ConfigKey) == {}:
            defaults = {'use_dhcp'   : False,
                        'use_dns'    : False,
                        'interface'  : 'eth0',
                        'lease_from' : '',
                        'lease_to'   : ''}

            for (key, val) in defaults.iteritems():
                models.ConfigDB.set(ConfigKey, key, val)

    #=======================================================
    # GET
    #=======================================================
    def handle_GET(self, *args, **kwargs):
        self.__module.log(syslog.LOG_INFO,
                "'GET' : args = %s, kwargs = %s" % (str(args), str(kwargs)))

        arg_nums = len(args)
        print "arg_nums = %d" % arg_nums

        if arg_nums == 0:
            conf = models.ConfigDB.select_by_kind(ConfigKey)
            return {'settings' : conf}

        raise cherrypy.HTTPError(404, "DHCPServer: request error.")

    #-----------------------------------
    # POST
    #-----------------------------------
    def handle_POST(self, *args, **kwargs):
        # args[0] <--- Interface (eth0)
        # kwargs  <--- command
        self.__module.log(syslog.LOG_INFO,
                "'POST' : args = %s, kwargs = %s" % (str(args), str(kwargs)))

        errcode = 0
        errmsg = ''
        arg_nums = len(args)

        if arg_nums == 0:
            if kwargs.has_key('command'):
                # コマンド処理
                return self.__POST_command(**kwargs)
            else:
                raise cherrypy.HTTPError(400, "DHCPServer: 'command' required.")
        else:
            raise cherrypy.HTTPError(400, "DHCPServer: Bad URL.")

    #---
    def __POST_command(self, **kwargs):
        command = kwargs['command']

        if command == 'write_config':
            self.__module.write_config()
        elif command == 'start':
            res = self.__daemon.start(ServiceName)
        elif command == 'stop':
            res = self.__daemon.stop(ServiceName)
        elif command == 'restart':
            res = self.__daemon.restart(ServiceName)
        else:
            raise cherrypy.HTTPError(400, "DHCPServer: Bad command '%s'" % command)

        if res == 0:
            message = "Command '%s' succeeded." % command
        else:
            message = "Command '%s' failed." % command

        return {'command' : command,
                'result'  : res,
                'message' : message}

    #-----------------------------------
    # PUT (change settings)
    #-----------------------------------
    def handle_PUT(self, *args, **kwargs):
        # args[0] <--- Interface (eth0)
        # kwargs  <--- configuration
        self.__module.log(syslog.LOG_INFO,
                "'PUT' : args = %s, kwargs = %s" % (str(args), str(kwargs)))

        keys = ('interface', 'use_dhcp', 'lease_from', 'lease_to')

        for key in keys:
            value = kwargs.get(key, None)

            if value != None:
                models.ConfigDB.set(ConfigKey, key, value)

        self.__module.write_data()

        return {'result': True}
    #-----------------------------------
    # Update daemon
    #-----------------------------------
    def update_daemon(self):
        # 有効 / 無効 → Daemontools
        dhcp_enable = models.ConfigDB.select_by_kind_key(ConfigKey, 'use_dhcp')
        dns_enable  = models.ConfigDB.select_by_kind_key(ConfigKey, 'use_dns')

        if dhcp_enable == True or dns_enable == True:
            self.__daemon.enable(ServiceName)
        else:
            self.__daemon.disable(ServiceName)


#-------------------------------------------------------------------------------
# Self Test
#-------------------------------------------------------------------------------
if __name__ == '__main__':
    class Empty(BaseModule.BaseModule):
        exposed = True
        def __init__(self):
            BaseModule.BaseModule.__init__(self)

        def write_config(self):
            return {'result'  : 0,
                    'message' : 'Test OK'}

    def get_password(username):
        return 'admin'

    print "## DHCPServer Module test"
    #res = Resource.Resource()
    #conf = Config.Config()

    dnsmasq = DHCPServer(res, conf)

    root = Empty()
    root.service = Empty()
    root.service.conf = Empty()
    root.service.conf.net = Empty()
    root.service.conf.net.dnsmasq = dnsmasq

    parent = ethernet.get_parentmodule()
    print "ethernet.get_parentmodule() --> %s" % str(parent)

    print "childmodule --> %s" % str(parent.get_childmodule_names())

    cp_conf = {'global' :
               {'request.show_tracebacks' : True},
               '/' : {'request.dispatch' : cherrypy.dispatch.MethodDispatcher(),
                      'tools.wsse_auth.on'           : True,
                      'tools.wsse_auth.realm'        : 'This Site',
                      'tools.wsse_auth.get_password' : get_password,

                      'tools.dejsonifier.on' : True,
                      'tools.jsonifier.on'   : True,}
               }

    cherrypy.quickstart(root, '/', config=cp_conf)
