#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# MA-4xx/8xx Web Application Server Module
#   Ethernet module
# Author : Takeyoshi Kikuchi <kikuchi@centurysys.co.jp>
#
# Copyright (C) 2007-2010 Century Systems Co.,Ltd.
# Last Modified: 2010/05/26 10:19:23 kikuchi
#

import copy
import syslog
import cherrypy

# Configuration DB
from DB import models

from Tools import jsonlib
from Tools import wsse_auth

from Utils import RCscript
from Utils import Utils

from Handlers import BaseModule

ConfigKey = "Ethernet"

DHCP_opt2val = {'release'   : 2 ** 0,
                'nodns'     : 2 ** 1,
                'nontp'     : 2 ** 2,
                'nogateway' : 2 ** 3}

DHCP_val2opt = {2 ** 0      : 'release',
                2 ** 1      : 'nodns',
                2 ** 2      : 'nontp',
                2 ** 3      : 'nogateway'}


#===============================================================================
#
# Ethernet module
#
#===============================================================================
class Ethernet(BaseModule.BaseModule):
    exposed = True

    def __init__(self, debug=False, prefix='output_test/'):
        # SuperClass (BaseModule)
        self.__super = BaseModule.BaseModule

        # モジュール内部データ管理クラス
        self.__ether = InternalData(self)

        # Superclass Constructor
        self.__super.__init__(self)

        self.log_header = "Ethernet"

    #-----------------------------------
    # HTTP method handler (GET)
    #-----------------------------------
    def GET(self, *args, **kwargs):
        return self.__ether.handle_GET(*args, **kwargs)

    #-----------------------------------
    # HTTP method handler (POST)
    #-----------------------------------
    def POST(self, *args, **kwargs):
        return self.__ether.handle_POST(*args, **kwargs)

    #-----------------------------------
    # HTTP method handler (PUT)
    #-----------------------------------
    def PUT(self, *args, **kwargs):
        return self.__ether.handle_PUT(*args, **kwargs)

    #-----------------------------------
    # HTTP method handler (DELETE)
    #-----------------------------------
    def DELETE(self, *args, **kwargs):
        return "Ok"

    #---------------------------------------------
    # 設定ファイル書きだし
    #---------------------------------------------
    def write_data(self):
        pass

    #---------------------------------------------
    # extra functions
    #---------------------------------------------
    def get_data(self):
        pass


#-------------------------------------------------------------------------------
# 内部データ管理
#-------------------------------------------------------------------------------
class InternalData:
    #-----------------------------------
    #
    #-----------------------------------
    def __init__(self, module):
        self.__rc = RCscript.RCscript()
        self.__config_init(module)

    #-----------------------------------
    #
    #-----------------------------------
    def __config_init(self, module):
        self.__module = module

        # 設定が無い場合のデフォルト
        if models.Ethernet.select_by_intf('eth0') == None:
            models.vEthernet.insert_or_update(
                    {'name' : 'eth0', 'address' : '192.168.253.253',
                       'prefixlen' : 24, 'type' : 0, 'erasable' : False})

    #=======================================================
    # GET
    #=======================================================
    def handle_GET(self, *args, **kwargs):
        self.__module.log(syslog.LOG_INFO, 
                "'GET' : *args = %s, **kwargs = %s" % (str(args), str(kwargs)))
        arg_nums = len(args)

        if arg_nums == 0:
            return self.__GET_interfaces()
        elif arg_nums == 1:
            return self.__GET_interface_info(args[0])

        raise cherrypy.HTTPError(404, "Ethernet: object not found.")

    #---
    def __GET_interfaces(self):
        list = []
        interfaces = models.vEthernet.select_all()
        for intf in interfaces:
            list.append(intf['name'])
        return {'Interfaces' : list}

    #---
    def __GET_interface_info(self, interface):
        # Get all interface information (ethernet only)
        if interface == 'all':
            infos = models.vEthernet.select_all()

            self.__module.log(syslog.LOG_INFO, "all --> %s" % str(infos))
            return {'all' : infos}
        else:
            eth_intf = models.vEthernet.select_by_intf(interface).get_dict()
            if eth_intf == None:
                raise cherrypy.HTTPError(
                        404, "Ethernet: '%s' isn't found." % interface)

            # DHCP ?
            if eth_intf.get('use_dhcp', False) == True:
                options = eth_intf.get('dhcp_opts', 0)
                eth_intf['dhcp_opts'] = self.__decode_dhcp_opts(options)
            else:
                eth_intf['dhcp_opts'] = []
            eth_intf['netmask'] = Utils.prefixlen2netmask(eth_intf['prefixlen'])

            return eth_intf

    #---
    #
    def __decode_dhcp_opts(self, options):
        dhcp_opts = []

        for (key, val) in DHCP_val2opt.iteritems():
            if options & key:
                dhcp_opts.append(val)

        return dhcp_opts

    #-----------------------------------
    # POST
    #-----------------------------------
    def handle_POST(self, *args, **kwargs):
        # args[0] <--- Interface (eth0)
        # kwargs  <--- command
        self.__module.log(syslog.LOG_INFO, 
                "'POST' : *args = %s, **kwargs = %s" % (str(args), str(kwargs)))

        errcode = 0
        errmsg = ''
        arg_nums = len(args)

        if arg_nums == 0:
            if kwargs.has_key('command'):
                command = kwargs['command']

                # 設定ファイルへ書き込み
                if command == 'write_config':
                    # (上位モジュールを呼び出す)
                    parent = self.__module.get_parentmodule()
                    res = parent.write_config(args[0])

                    return {'command' : command,
                            'result'  : res['result'],
                            'message' : res['message']}
                else:
                    # HTTP Error
                    errcode = 400
                    errmsg = "Ethernet: Bad command '%s'" % command
            else:
                # HTTP Error
                errcode = 400
                errmsg = "Ethernet: 'command' required."
        elif arg_nums != 1:
            # HTTP Error
            errcode = 404
            errmsg = "Ethernet: interface required."

        if errcode == 0:
            interface = args[0]
            eths = models.Ethernet.select_all()

            interfaces = []
            for eth in eths:
                interfaces.append(eth['name'])

            if not interface in interfaces:
                # HTTP Error
                errcode = 404
                errmsg = "Ethernet: '%s' isn't found." % interface
            elif not kwargs.has_key('command'):
                # HTTP Error
                errcode = 400
                errmsg = "Ethernet: 'command' required."
            else:
                return self.__POST_command(interface, **kwargs)

        raise cherrypy.HTTPError(errcode, errmsg)

    #---
    def __POST_command(self, interface, **kwargs):
        command = kwargs['command']

        script = 'net.' + interface

        if command == 'start':
            res = self.__rc.start(script)
        elif command == 'stop':
            res = self.__rc.stop(script)
        elif command == 'restart':
            res = self.__rc.restart(script)
        else:
            raise cherrypy.HTTPError(400, "Ethernet: Bad command '%s'" % command)

        if res == 0:
            message = "Command '%s' succeeded." % command
        else:
            message = "Command '%s' failed." % command

        return {'command' : command,
                'result'  : res,
                'message' : message}

    #-----------------------------------
    # PUT (change settings)
    #-----------------------------------
    def handle_PUT(self, *args, **kwargs):
        # args[0] <--- Interface (eth0)
        # kwargs  <--- configuration
        self.__module.log(syslog.LOG_INFO, 
                "'PUT' : *args = %s, **kwargs = %s" % (str(args), str(kwargs)))

        prefixlen = Utils.netmask2prefixlen(str(kwargs['netmask']))
        intf = args[0]

        dhcp_opts = 0
        if kwargs['use_dhcp'] == True:
            for opts in kwargs['dhcp_opts']:
                dhcp_opts += DHCP_opt2val.get(opts, 0)

        dict_args = {'name'      : intf,
                     'address'   : kwargs['address'],
                     'prefixlen' : prefixlen,
                     'gateway'   : kwargs['gateway'],
                     'use_dhcp'  : kwargs['use_dhcp'],
                     'dhcp_opts' : dhcp_opts}

        result = models.Ethernet.insert_or_update(dict_args)

        # 上位モジュールで書き込み
        parent = self.__module.get_parentmodule()
        parent.write_data(intf)

        return {'result': True}

        
#-------------------------------------------------------------------------------
# Self Test
#-------------------------------------------------------------------------------
if __name__ == '__main__':
    class Empty(BaseModule.BaseModule):
        exposed = True
        def __init__(self):
            BaseModule.BaseModule.__init__(self)

        def write_config(self):
            return {'result'  : 0,
                    'message' : 'Test OK'}

    def get_password(username):
        return 'admin'

    print "## Ethernet Module test"
    #res = Resource.Resource()
    #conf = Config.Config()

    ethernet = Ethernet(res, conf)

    ethernet.get_data()

    root = conf.get_item('root')

    import yaml
    print yaml.safe_dump(root)

    eth = conf.get_item('Ethernet')

    print yaml.safe_dump(conf.get_item('root'))

    root = conf.get_item('root')

    conf = None

    print str(root)

    conf2 = Config.Config(root)

    print yaml.safe_dump(conf2.get_item('root'))

    # /conf/net/ethernet
    root = Empty()
    root.service = Empty()
    root.service.conf = Empty()
    root.service.conf.net = Empty()
    root.service.conf.net.ethernet = ethernet

    parent = ethernet.get_parentmodule()
    print "ethernet.get_parentmodule() --> %s" % str(parent)

    print "childmodule --> %s" % str(parent.get_childmodule_names())

    cp_conf = { 'global' :
                { 'request.show_tracebacks' : True },
                '/' :
                { 'request.dispatch' : cherrypy.dispatch.MethodDispatcher(),

                  'tools.wsse_auth.on'           : True,
                  'tools.wsse_auth.realm'        : 'This Site',
                  'tools.wsse_auth.get_password' : get_password,

                  'tools.dejsonifier.on' : True,
                  'tools.jsonifier.on'   : True,
                  } }

    cherrypy.quickstart(root, '/', config = cp_conf)
