#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# MA-4xx/8xx/E2xx Web Application Server Module
#   NTP module
# Author : Takeyoshi Kikuchi <kikuchi@centurysys.co.jp>
#
# Copyright (C) 2007-2008 Century Systems Co.,Ltd.
# Last Modified: 2010/05/26 09:24:27 kikuchi
#

import copy
import cherrypy
import syslog

# Configuration DB
from DB import config, models

from Tools import jsonlib
from Tools import wsse_auth

from Utils import RCscript
from Utils import DaemonTools

from Handlers import BaseModule

# TemplateModule FrontEnd
from Templates.Template_ntp import Template_ntp

ConfigKey   = "NTP"
ServiceName = "ntpd"


#===============================================================================
#
# NTP module
#
#===============================================================================
class NTP(BaseModule.BaseModule):
    exposed = True

    def __init__(self, debug = False, prefix = 'output_test/'):
        # SuperClass (BaseModule)
        self.__super = BaseModule.BaseModule

        # モジュール内部データ管理クラス
        self.__ntp = InternalData(self)

        # Template FrontEnd
        self.__template = Template_ntp(debug, prefix)

        # Debug / Production
        self.__debug = debug

        # Superclass Constructor
        self.__super.__init__(self)

        self.log_header = "NTP"

    #-----------------------------------
    # HTTP method handler (GET)
    #-----------------------------------
    def GET(self, *args, **kwargs):
        return self.__ntp.handle_GET(*args, **kwargs)

    #-----------------------------------
    # HTTP method handler (POST)
    #-----------------------------------
    def POST(self, *args, **kwargs):
        return self.__ntp.handle_POST(*args, **kwargs)

    #-----------------------------------
    # HTTP method handler (PUT)
    #-----------------------------------
    def PUT(self, *args, **kwargs):
        return self.__ntp.handle_PUT(*args, **kwargs)

    #-----------------------------------
    # HTTP method handler (DELETE)
    #-----------------------------------
    def DELETE(self, *args, **kwargs):
        raise cherrypy.HTTPError('400', 'NTP: Bad Request.')

    #---------------------------------------------
    # 設定ファイル書きだし
    #---------------------------------------------
    def write_data(self):
        data = models.ConfigDB.select_by_kind(ConfigKey)
        self.log(syslog.LOG_INFO, "write_data(): confData = %s" % str(data))

        self.__template.write_config({'NTP' : {'ntp_info' : data}})

        # DaemonTools 設定更新
        if self.__debug == False:
            # when production only
            self.__ntp.update_daemon(data)


#-------------------------------------------------------------------------------
# 内部データ管理
#-------------------------------------------------------------------------------
class InternalData:
    #-----------------------------------
    # Constructor
    #-----------------------------------
    def __init__(self, module):
        self.__module = module

        self.__daemon   = DaemonTools.DaemonTools()
        self.__rc       = RCscript.RCscript()
        self.__rcscript = "ntp-client"

        if models.ConfigDB.select_by_kind(ConfigKey) == {}:
            # default
            models.ConfigDB.set(ConfigKey, 'use_ntp', False)
            models.ConfigDB.set(ConfigKey, 'act_server', False)
            models.ConfigDB.set(ConfigKey, 'ntp_server_primary', 'ntp.nict.jp')
            models.ConfigDB.set(ConfigKey, 'ntp_server_secondary', 'pool.ntp.org')

    #=======================================================
    # GET (HTTP Method handler)
    #=======================================================
    def handle_GET(self, *args, **kwargs):
        # **kwargs is ignored
        self.__module.log(syslog.LOG_INFO,
            "'GET' : *args = %s, **kwargs = %s" % (str(args), str(kwargs)))

        ntp_info = models.ConfigDB.select_by_kind(ConfigKey)

        return {'ntp_info' : ntp_info}

    #=======================================================
    # POST (HTTP Method handler)
    #=======================================================
    def handle_POST(self, *args, **kwargs):
        self.__module.log(syslog.LOG_INFO, 
            "'POST' : *args = %s, **kwargs = %s" % (str(args), str(kwargs)))

        arg_nums = len(args)

        if arg_nums == 0:
            if kwargs.has_key('command'):
                command = kwargs['command']

                # Reboot NTP Server
                if command == 'reboot':
                    self.__daemon.restart(ServiceName)
                    return { 'command' : command,
                             'result'  : True }
                else:
                    raise cherrypy.HTTPError(400, "NTP: Bad command '%s'" % command)
            else:
                raise cherrypy.HTTPError(400, "NTP: 'command' required")
        else:
            raise cherrypy.HTTPError(400, "NTP: Bad URL.")

    #=======================================================
    # PUT
    #=======================================================
    def handle_PUT(self, *args, **kwargs):
        self.__module.log(syslog.LOG_INFO,
            "'PUT' : *args = %s, **kwargs = %s" % (str(args), str(kwargs)))

        keys = ('use_ntp', 'act_server',
                'ntp_server_primary',
                'ntp_server_secondary')
        try:
            for key in keys:
                models.ConfigDB.set(ConfigKey, key, kwargs[key])

            self.__module.write_data()

            #self.update_daemon(__confData)

            return {'result': True}

        except Exception, msg:
            self.__module.log(syslog.LOG_ERR, "PUT: Error '%s'" % str(msg))
            raise cherrypy.HTTPError('400', 'NTP: Bad Request, %s' % str(msg))

    #-----------------------------------
    # Update daemon
    #-----------------------------------
    def update_daemon(self, __confData):
        # ntpd 有効/無効
        self.__module.log(syslog.LOG_INFO,
            "update_daemon() __confData = %s" % str(__confData))

        enable = models.ConfigDB.select_by_kind_key(ConfigKey, 'use_ntp')
        if enable == None:
            enable = False
            models.ConfigDB.set(ConfigKey, 'use_ntp', False)

        if enable == True:
            self.__module.log(syslog.LOG_INFO, 
                    "rc-update add %s default" % self.__rcscript)
            self.__rc.add_default(self.__rcscript)
            self.__daemon.enable(ServiceName)
        else:
            self.__module.log(syslog.LOG_INFO, 
                "rc-update del %s default" % self.__rcscript)
            self.__rc.del_default(self.__rcscript)
            self.__daemon.disable(ServiceName)
    

#-------------------------------------------------------------------------------
# Self Test
#-------------------------------------------------------------------------------
if __name__ == '__main__':
    class Empty(BaseModule.BaseModule):
        exposed = True
        def __init__(self):
            BaseModule.BaseModule.__init__(self)

        def write_config(self):
            return {'result'  : 0,
                    'message' : 'Test OK'}

    def get_password(username):
        return 'password1'

    print "## DateTime Module test"
    #res = Resource.Resource()
    #conf = Config.Config()

    DateTime = DateTime(res, conf)

    # /conf/datetime
    root = Empty()
    root.conf = Empty()
    root.conf.datetime = DateTime

    cp_conf = {'global' :
               {'request.show_tracebacks' : False},
               '/' :
               {'request.dispatch' : cherrypy.dispatch.MethodDispatcher(),

                'tools.wsse_auth.on'           : True,
                'tools.wsse_auth.realm'        : 'This Site',
                'tools.wsse_auth.get_password' : get_password,

                'tools.dejsonifier.on' : True,
                'tools.jsonifier.on'   : True,
                }}
    
    cherrypy.quickstart(root, '/', config=cp_conf)
