#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# MA-4xx/8xx/E2xx Web Application Server Module
#   Network module
# Author : Takeyoshi Kikuchi <kikuchi@centurysys.co.jp>
#
# Copyright (C) 2007-2008 Century Systems Co.,Ltd.
# Last Modified: 2010/05/26 10:19:31 kikuchi
#

import cherrypy
import syslog

# Configuration DB
from DB import config, models

from Tools import jsonlib
from Tools import wsse_auth

from Utils import RCscript
from Utils import Utils

from Handlers import BaseModule

# TemplateModule FrontEnd
from Templates.Template_Net import Template_Net

ConfigKey = "Network"


#===============================================================================
#
# Network module
#
#===============================================================================
class Network(BaseModule.BaseModule):
    exposed = True

    def __init__(self, debug=False, prefix='output_test/'):
        # SuperClass (BaseModule)
        self.__super = BaseModule.BaseModule

        # モジュール内部データ管理クラス
        self.__ether = InternalData(self)

        # Template FrontEnd
        self.__template = Template_Net(debug, prefix)

        # Superclass Constructor
        self.__super.__init__(self)

        self.log_header = "Network"

    #-----------------------------------
    # HTTP method handler (GET)
    #-----------------------------------
    def GET(self, *args, **kwargs):
        return self.__ether.handle_GET(*args, **kwargs)

    #-----------------------------------
    # HTTP method handler (POST)
    #-----------------------------------
    def POST(self, *args, **kwargs):
        return self.__ether.handle_POST(*args, **kwargs)

    #-----------------------------------
    # HTTP method handler (PUT)
    #-----------------------------------
    def PUT(self, *args, **kwargs):
        return self.__ether.handle_PUT(*args, **kwargs)

    #-----------------------------------
    # HTTP method handler (DELETE)
    #-----------------------------------
    def DELETE(self, *args, **kwargs):
        return "Ok"

    #------------------------------------------------
    # 設定ファイル書きだし
    # caller: None      config復帰時
    #         eth0      ethernet画面更新時
    #         gre       greモジュールからの呼び出し
    #         route     routeモジュールからの呼び出し 
    #------------------------------------------------
    def write_data(self, caller=None):
        # Ethernet / Route をまとめる必要あり
        conf = {}
        conf['Ethernet']  = models.vEthernet.select_all()
        conf['Route']     = models.Route.select_all()
        conf['DNSServer'] = models.ConfigDB.select_by_kind('DNSServer')
        conf['GRE']       = models.vGre.select_all()

        self.__template.write_config(conf)

        # ethernet I/Fのみ再起動
        if caller == None:
            self.__ether.restartInterface("eth0")
        elif caller in ("eth0"):
            self.__ether.restartInterface(caller)
        else:
            pass

    #---------------------------------------------
    # extra functions
    #---------------------------------------------
    def get_data(self):
        print str(self.confData)


#-------------------------------------------------------------------------------
# 内部データ管理
#-------------------------------------------------------------------------------
class InternalData:
    #-----------------------------------
    #
    #-----------------------------------
    def __init__(self, module):
        self.__module = module
        self.__rc = RCscript.RCscript()

    #=======================================================
    # GET
    #=======================================================
    def handle_GET(self, *args, **kwargs):
        self.__module.log(syslog.LOG_INFO,
                "'GET': args = %s, kwargs = %s" % (str(args), str(kwargs)))
        # **kwargs is ignored
        arg_nums = len(args)

        if arg_nums == 0:
            return self.__GET_interfaces()
        elif arg_nums == 1:
            return self.__GET_interface_info(args[0])

        raise cherrypy.HTTPError(404, "Network: object not found.")

    #---
    def __GET_interfaces(self):
        # Get all interface information (includes PPP)
        intfs = models.Interface.select_all()

        info = []
        for intf in intfs:
            if intf['type'] == 0:
                # Ethernet
                try:
                    eth_conf = models.vEthernet.select_by_intf(intf['name']).get_dict()
                    eth_conf['netmask'] = Utils.prefixlen2netmask(eth_conf['prefixlen'])

                    info.append({'interface' : intf['name'],
                                 'type'      : 'Ethernet',
                                 'config'    : eth_conf})
                except Exception, msg:
                    self.__module.log(syslog.LOG_ERR,
                            "__GET_interfaces() error: '%s'" % str(msg))

            elif intf['type'] == 1:
                # PPP
                try:
                    ppp_conf = models.vPPP.select_by_intf(str(intf['name'])).get_dict()
                    if ppp_conf != None:
                        info.append({'interface' : intf['name'],
                                     'type'      : 'PPP',
                                     'config'    : ppp_conf})
                except Exception, msg:
                    self.__module.log(syslog.LOG_ERR,
                            "__GET_interfaces() error: '%s'" % str(msg))

            elif intf['type'] == 2:
                # GRE
                try:
                    gre_conf = models.vGre.select_by_intf(str(intf['name'])).get_dict()
                    if gre_conf != None:
                        info.append({'interface' : intf['name'],
                                     'type'      : 'GRE',
                                     'config'    : gre_conf})
                except Exception, msg:
                    self.__module.log(syslog.LOG_ERR,
                            "__GET_interfaces() error: '%s'" % str(msg))

        # DialIn
        try:
            DialIn = models.DialIn.select_all()[0]
            DialIn_enable = DialIn.get('enable', False)

            if DialIn_enable:
                DialIn['conn_name'] = u'着信'

                info.append({'interface' : DialIn.get('name', 'ppp50'),
                             'type'      : 'PPP_In',
                             'config'    : DialIn})
        except Exception, msg:
            self.__module.log(syslog.LOG_ERR,
                    "__GET_interfaces() error: '%s'" % str(msg))

        self.__module.log(syslog.LOG_INFO,
                    "all --> %s" % str(info))
        return {'all' : info}

    #---
    #
    def __GET_interface_info(self, interface):
        if_info = models.Interface.select_by_intf(interface)

        if if_info == None:
            raise cherrypy.HTTPError(404, "Network: '%s' isn't found." % interface)

        return if_info

    #-----------------------------------
    # POST
    #-----------------------------------
    def handle_POST(self, *args, **kwargs):
        # args[0] <--- Interface (eth0)
        # kwargs  <--- command
        self.__module.log(syslog.LOG_INFO,
                "'POST': args = %s, kwargs = %s" % (str(args), str(kwargs)))

        errcode = 0
        errmsg = ''
        arg_nums = len(args)

        if arg_nums == 0:
            if kwargs.has_key('command'):
                command = kwargs['command']

                # 設定ファイルへ書き込み
                if command == 'write_config':
                    # (上位モジュールを呼び出す)
                    parent = self.__module.get_parentmodule()
                    res = parent.write_config()

                    return {'command' : command,
                            'result'  : res['result'],
                            'message' : res['message']}
                else:
                    # HTTP Error
                    errcode = 400
                    errmsg = "Network: Bad command '%s'" % command
            else:
                # HTTP Error
                errcode = 400
                errmsg = "Network: 'command' required."
        elif arg_nums != 1:
            # HTTP Error
            errcode = 404
            errmsg = "Network: interface required."

        if errcode == 0:
            interface = args[0]
            interfaces = self.__res.find_all_entries(ResourceKey)

            if not interface in interfaces:
                # HTTP Error
                errcode = 404
                errmsg = "Network: '%s' isn't found." % interface
            elif not kwargs.has_key('command'):
                # HTTP Error
                errcode = 400
                errmsg = "Network: 'command' required."
            else:
                return self.__POST_command(interface, **kwargs)

        raise cherrypy.HTTPError(errcode, errmsg)

    #---
    def __POST_command(self, interface, **kwargs):
        command = kwargs['command']

        script = 'net.' + interface

        if command == 'start':
            res = self.__rc.start(script)
        elif command == 'stop':
            res = self.__rc.stop(script)
        elif command == 'restart':
            res = self.__rc.restart(script)
        else:
            raise cherrypy.HTTPError(400, "Network: Bad command '%s'" % command)

        if res == 0:
            message = "Command '%s' succeeded." % command
        else:
            message = "Command '%s' failed." % command

        return {'command' : command,
                'result'  : res,
                'message' : message}

    #-----------------------------------
    # PUT (change settings)
    #-----------------------------------
    def handle_PUT(self, *args, **kwargs):
        # args[0] <--- Interface (eth0)
        # kwargs  <--- configuration
        self.__module.log(syslog.LOG_INFO,
                "'PUT': args = %s, kwargs = %s" % (str(args), str(kwargs)))

    def restartInterface(self, interface):
        script = 'net.' + interface
        res = self.__rc.restart(script)

#-------------------------------------------------------------------------------
# Self Test
#-------------------------------------------------------------------------------
if __name__ == '__main__':
    class Empty(BaseModule.BaseModule):
        exposed = True
        def __init__(self):
            BaseModule.BaseModule.__init__(self)

        def write_config(self):
            return {'result'  : 0,
                    'message' : 'Test OK'}

    def get_password(username):
        return 'admin'

    print "## Network Module test"
    #res = Resource.Resource()
    #conf = Config.Config()

    ethernet = Network(res, conf)

    ethernet.get_data()

    root = conf.get_item('root')

    import yaml
    print yaml.safe_dump(root)
    
    eth = conf.get_item('Network')

    print yaml.safe_dump(conf.get_item('root'))

    root = conf.get_item('root')

    conf = None

    print str(root)

    conf2 = Config.Config(root)

    print yaml.safe_dump(conf2.get_item('root'))

    # /conf/net/ethernet
    root = Empty()
    root.service = Empty()
    root.service.conf = Empty()
    root.service.conf.net = Empty()
    root.service.conf.net.ethernet = ethernet

    parent = ethernet.get_parentmodule()
    print "ethernet.get_parentmodule() --> %s" % str(parent)

    print "childmodule --> %s" % str(parent.get_childmodule_names())

    cp_conf = {'global' :
               {'request.show_tracebacks' : True},
               '/' :
               {'request.dispatch' : cherrypy.dispatch.MethodDispatcher(),

                'tools.wsse_auth.on'           : True,
                'tools.wsse_auth.realm'        : 'This Site',
                'tools.wsse_auth.get_password' : get_password,

                'tools.dejsonifier.on' : True,
                'tools.jsonifier.on'   : True,
                }}
    
    cherrypy.quickstart(root, '/', config=cp_conf)
