#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# MA-4xx/8xx/E2xx Web Application Server Module
#   OCN IPv6 module
# Author : Takeyoshi Kikuchi <kikuchi@centurysys.co.jp>
#
# Copyright (C) 2007-2008 Century Systems Co.,Ltd.
# Last Modified: 2010/05/26 09:24:56 kikuchi
#

import copy
import re
import cherrypy
import syslog
import pyiproute

# Configuration DB
from DB import config, models

from Tools import jsonlib
from Tools import wsse_auth

from Utils import RCscript
from Utils import DaemonTools

from Handlers import BaseModule

# TemplateModule FrontEnd
from Templates.Template_OCNIPv6 import Template_OCNIPv6

ConfigKey = "OCNv6"

StatFile = "/tmp/OCNIPv6_status"
PPP_unit = 100

#===============================================================================
#
# OCN IPv6 module
#
#===============================================================================
class OCNv6(BaseModule.BaseModule):
    exposed = True

    def __init__(self, debug=False, prefix='output_test/'):
        self.__super = BaseModule.BaseModule

        # モジュール内部データ管理クラス
        self.__ocnv6 = InternalData(self)

        # Template FrontEnd
        self.__template = Template_OCNIPv6(debug, prefix)

        # Debug / Production
        self.__debug = debug

        # Superclass Constructor
        self.__super.__init__(self)

        self.log_header = "OCNv6"

    #-----------------------------------
    # HTTP method handler (GET)
    #-----------------------------------
    def GET(self, *args, **kwargs):
        return self.__ocnv6.handle_GET(*args, **kwargs)

    #-----------------------------------
    # HTTP method handler (POST)
    #-----------------------------------
    def POST(self, *args, **kwargs):
        return self.__ocnv6.handle_POST(*args, **kwargs)

    #-----------------------------------
    # HTTP method handler (PUT)
    #-----------------------------------
    def PUT(self, *args, **kwargs):
        return self.__ocnv6.handle_PUT(*args, **kwargs)

    #-----------------------------------
    # HTTP method handler (DELETE)
    #-----------------------------------
    def DELETE(self, *args, **kwargs):
        raise cherrypy.HTTPError('400', 'DateTime: Bad Request.')

    #---------------------------------------------
    # 設定ファイル書きだし
    #---------------------------------------------
    def write_data(self):
        try:
            ocnv6   = models.vOCNIPv6.select_all()[0]
        except IndexError:
            self.log(syslog.LOG_ERR, "!!! Error: no OCN IPv6 setting found.")
            return

        secrets = models.Secret.select_all()
        self.__template.write_config({'OCNv6'   : ocnv6,
                                      'Secrets' : secrets})

        # Daemon 設定変更
        if self.__debug == False:
            # when production only
            self.__ocnv6.update_daemon()


#-------------------------------------------------------------------------------
# 内部データ管理
#-------------------------------------------------------------------------------
class InternalData:
    #-----------------------------------
    #
    #-----------------------------------
    def __init__(self, module):
        self.__module = module

        self.__ipaddr = pyiproute.ipaddr()

        self.__rc = RCscript.RCscript()
        self.__daemon = DaemonTools.DaemonTools()
        self.__script = 'ocnv6'

    #=======================================================
    # GET
    #=======================================================
    def handle_GET(self, *args, **kwargs):
        # **kwargs is ignored
        self.__module.log(syslog.LOG_INFO, 
                "'GET' : *args = %s, **kwargs = %s" % (str(args), str(kwargs)))

        if len(args) == 1:
            if args[0] == 'settings':
                # アカウント情報
                info = models.vOCNIPv6.select_all()

                if len(info) == 1:
                    return info[0]
            if args[0] == 'status':
                # 接続状態
                info = self.get_status()
                self.__module.log(syslog.LOG_INFO, "status: %s" % str(info))

                result = {}
                if info.has_key('prefix'):
                    result['connection'] = True
                    result['prefix']     = info['prefix']
                    result['prefixlen']  = int(info['prefixlen'])
                else:
                    result['connection'] = False
                    result['prefix']     = ''
                    result['prefixlen']  = 128
                    if info.has_key('action'):
                        result['action'] = info['action']
                    
                return result

    #=======================================================
    # POST
    #=======================================================
    def handle_POST(self, *args, **kwargs):
        # **kwargs is ignored
        self.__module.log(syslog.LOG_INFO, 
            "'POST' : *args = %s, **kwargs = %s" % (str(args), str(kwargs)))

        arg_nums = len(args)

        if arg_nums == 0:
            if kwargs.has_key('command'):
                status = self.get_status()

                command = kwargs['command']
                
                # 接続
                if command == 'connect':
                    self.__update_stat("Connect")
                    res = self.__rc.start(self.__script)
                # 切断
                elif command == 'disconnect':
                    if status.has_key('action'):
                        if status['action'] == 'connect':
                            # 恐らくアカウントの間違いなどで接続中のまま
                            self.__update_stat("Down")
                    else:
                        self.__update_stat("Disconnect")
                    res = self.__rc.stop(self.__script)
                # 切断→接続
                elif command == 'reconnect':
                    self.__update_stat("Connect")
                    res = self.__rc.restart(self.__script)
                else:
                    raise cherrypy.HTTPError(400, "OCNv6: Bad command '%s'" % command)

                if res == 0:
                    message = "Command '%s' succeeded." % command
                else:
                    message = "Command '%s' failed." % command

                return {'command' : command,
                        'result'  : res,
                        'message' : message}
            else:
                raise cherrypy.HTTPError(400, "OCNv6: command required")
        else:
            raise cherrypy.HTTPError(400, "OCNv6: Bad Request URL")

    #---
    #
    def __update_stat(self, action):
        fd = open(StatFile, 'w+')

        if action == 'Connect':
            fd.write("CONNECTING\n")
        elif action == 'Disconnect':
            fd.write("DISCONNECTING\n")
        elif action == 'Down':
            fd.write("DOWN\n")

        fd.close()

    #=======================================================
    # PUT
    #=======================================================
    def handle_PUT(self, *args, **kwargs):
        self.__module.log(syslog.LOG_INFO,
            "'PUT' : *args = %s, **kwargs = %s" % (str(args), str(kwargs)))

        if len(args) == 1:
            if args[0] == 'settings':
                models.vOCNIPv6.insert_or_update(kwargs)
                self.__module.write_data()
                return {'result' : True}

        raise cherrypy.HTTPError(400, "OCNv6: Bad Request")

    #-----------------------------------
    # Update daemon
    #-----------------------------------
    def update_daemon(self):
        try:
            conf = models.vOCNIPv6.select_all()[0]
            # 起動時自動接続 ?
            if conf['auto_connect'] == True:
                self.__module.log(syslog.LOG_INFO,
                        "rc-update add %s default" % self.__script)
                self.__rc.add_default(self.__script)
            else:
                self.__module.log(syslog.LOG_INFO,
                        "rc-update del %s default" % self.__script)
                self.__rc.del_default(self.__script)
            
        except Exception, err:
            self.__module.log(syslog.LOG_ERR,
                "update_daemon() error: '%s'" % str(err))

    #=======================================================
    # check connection status
    #=======================================================
    def get_status(self):
        info = {}

        try:
            fd = open(StatFile, 'r')
            buf = fd.read()
            fd.close()

            hexstr = "[0-9a-f]"
            pattern = ("(UP|DOWN|CONNECTING|DISCONNECTING)"
                       "( (%s+:%s+:%s+:%s+)::/(64))*"
                       % (hexstr, hexstr, hexstr, hexstr))

            match = re.match(pattern, buf)

            if match != None:
                groups = match.groups()

                if groups[0] == 'DOWN':
                    self.__module.log(syslog.LOG_INFO, "OCN IPv6 is DOWN")
                    info = {'connection' : False}
                elif groups[0] == 'UP':
                    self.__module.log(syslog.LOG_INFO,
                        "OCN IPv6 is UP, prefix = '%s'" % groups[1])
                    info = {'connection' : True,
                            'prefix' : groups[2],
                            'prefixlen' : groups[3]}
                elif groups[0] == 'CONNECTING':
                    self.__module.log(syslog.LOG_INFO, "Connecting to OCN IPv6...")
                    info = {'connection' : False,
                            'action'     : 'connect'}
                elif groups[0] == 'DISCONNECTING':
                    self.__module.log(syslog.LOG_INFO,
                            "Disconnecting from OCN IPv6...")
                    info = {'connection' : False,
                            'action'     : 'disconnect'}
        except IOError:
            pass

        if info == {}:
            info = {'connection' : False}

        return info


#-------------------------------------------------------------------------------
# Self Test
#-------------------------------------------------------------------------------
if __name__ == '__main__':
    class Empty(BaseModule.BaseModule):
        exposed = True
        def __init__(self):
            BaseModule.BaseModule.__init__(self)

        def write_config(self):
            return {'result'  : 0,
                    'message' : 'Test OK'}

    def get_password(username):
        return 'admin'

    print "## OCN IPv6 Module test"
    #res = Resource.Resource()
    #conf = Config.Config()

    ocnv6 = OCNv6(res, conf)

    # /conf/datetime
    root = Empty()
    root.conf = Empty()
    root.conf.net = Empty()
    root.conf.net.ocnv6 = ocnv6

    cp_conf = {'global' :
               {'request.show_tracebacks' : False},
               '/' :
               {'request.dispatch' : cherrypy.dispatch.MethodDispatcher(),

                'tools.wsse_auth.on'           : True,
                'tools.wsse_auth.realm'        : 'This Site',
                'tools.wsse_auth.get_password' : get_password,

                'tools.dejsonifier.on' : True,
                'tools.jsonifier.on'   : True,
                }}
    
    cherrypy.quickstart(root, '/', config=cp_conf)
