#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# MA-4xx/8xx/E2xx Web Application Server Module
#   PPP(dialin) module
# Author : Takeyoshi Kikuchi <kikuchi@centurysys.co.jp>
#
# Copyright (C) 2008 Century Systems Co.,Ltd.
# Last Modified: 2010/05/26 09:26:04 kikuchi
#

import copy
import re
import cherrypy
import syslog
import glob

# Configuration DB
from DB import config, models

from Tools import jsonlib
from Tools import wsse_auth

from Utils import PPPstat
from Utils import DaemonTools

from Handlers import BaseModule

# TemplateModule FrontEnd
from Templates.Template_PPP_dialin import Template_PPP_dialin

ConfigKey = "PPP_dialin"
ServiceName = "mgetty"


#===============================================================================
#
# PPP(dialin) module
#
#===============================================================================
class PPP_dialin(BaseModule.BaseModule):
    exposed = True

    def __init__(self, debug=False, prefix='output_test/'):
        self.__super = BaseModule.BaseModule

        # モジュール内部データ管理クラス
        self.__dialin = InternalData(self)

        # Template FrontEnd
        self.__template = Template_PPP_dialin(debug, prefix)

        # Debug / Production
        self.__debug = debug

        # Superclass Constructor
        self.__super.__init__(self)

        self.log_header = "PPP_dialin"

    #-----------------------------------
    # HTTP method handler (GET)
    #-----------------------------------
    def GET(self, *args, **kwargs):
        return self.__dialin.handle_GET(*args, **kwargs)

    #-----------------------------------
    # HTTP method handler (POST)
    #-----------------------------------
    def POST(self, *args, **kwargs):
        return self.__dialin.handle_POST(*args, **kwargs)

    #-----------------------------------
    # HTTP method handler (PUT)
    #-----------------------------------
    def PUT(self, *args, **kwargs):
        return self.__dialin.handle_PUT(*args, **kwargs)

    #-----------------------------------
    # HTTP method handler (DELETE)
    #-----------------------------------
    def DELETE(self, *args, **kwargs):
        return self.__dialin.handle_DELETE(*args, **kwargs)

    #---------------------------------------------
    # 設定ファイル書きだし
    #---------------------------------------------
    def write_data(self):
        try:
            ppp_dialin = models.DialIn.select_all()[0]
        except IndexError:
            return

        secrets    = models.Secret.select_all()
        ntp_info   = models.ConfigDB.select_by_kind('NTP')
        dhcp_info  = models.ConfigDB.select_by_kind('DHCPServer')

        self.__template.write_config({'PPP_dialin' : ppp_dialin,
                                      'Secrets'    : secrets,
                                      'NTP'        : {'ntp_info'  : ntp_info},
                                      'DHCPServer' : {'dhcp_info' : dhcp_info}})

        # Daemon 設定変更
        if self.__debug == False:
            # when production only
            self.__dialin.update_daemon()


#-------------------------------------------------------------------------------
# 内部データ管理
#-------------------------------------------------------------------------------
class InternalData:
    #-----------------------------------
    #
    #-----------------------------------
    def __init__(self, module):
        self.__module = module
        self.__daemon = DaemonTools.DaemonTools()

    #=======================================================
    # GET
    #=======================================================
    def handle_GET(self, *args, **kwargs):
        # **kwargs is ignored
        self.__module.log(syslog.LOG_INFO, 
                "'GET' : *args = %s, **kwargs = %s" % (str(args), str(kwargs)))

        if len(args) == 1:
            if args[0] == 'devices':
                # get available devices list
                devices = PPPstat.get_available_devices(dialin=True)

                return {'devices' : devices}
            elif args[0] == 'status' or args[0] == 'conf':
                # get PPP status (ALL)
                status = models.DialIn.select_all()[0]

                return {args[0] : status}
            elif args[0] == 'accounts':
                # get PPP status (ALL)
                accounts = models.Secret.select_by_unit(50)

                return {'accounts' : accounts}
        else:
            raise cherrypy.HTTPError('400', 'PPP_dialin: Bad Request.')

    #=======================================================
    # POST
    #=======================================================
    def handle_POST(self, *args, **kwargs):
        # **kwargs is ignored
        self.__module.log(syslog.LOG_INFO, 
                "'POST' : *args = %s, **kwargs = %s" % (str(args), str(kwargs)))

        raise cherrypy.HTTPError('400', 'PPP_dialin: Bad Request.')

    #=======================================================
    # PUT
    #=======================================================
    def handle_PUT(self, *args, **kwargs):
        self.__module.log(syslog.LOG_INFO, 
                "'PUT' : *args = %s, **kwargs = %s" % (str(args), str(kwargs)))

        result = None

        if len(args) == 1:
            if args[0] == 'conf':
                # mgetty 設定更新
                try:
                    # 着信Enable --> dnsmasq enable
                    dialin_enable = kwargs['enable']
                    models.ConfigDB.set('DHCPServer', 'use_dns', dialin_enable)
                    
                    res = models.DialIn.insert_or_update(kwargs)

                    result = {'result' : res[0],
                              'msg'    : str(res[1])}
                except Exception, msg:
                    self.__module.log(syslog.LOG_ERR, "PUT Error: %s" % str(msg))
                    return {'result' : False,
                            'msg'    : str(msg)}
            elif args[0] == 'accounts':
                # mgetty アカウント更新
                accounts = kwargs['accounts']

                # DialIn アカウント削除
                models.Secret.delete_by_unit(50)

                for entry in accounts:
                    models.Secret.insert({'unit'     : 50,
                                          'account'  : entry['account'],
                                          'password' : entry['password']})
                result = {'result' : True}

        if result != None:
            self.__module.write_data()
            return result

        raise cherrypy.HTTPError('400', 'PPP_dialin: Bad Request.')

    #=======================================================
    # DELETE
    #=======================================================
    def handle_DELETE(self, *args, **kwargs):
        self.__module.log(syslog.LOG_INFO, 
                "'DELETE' : *args = %s, **kwargs = %s" % (str(args), str(kwargs)))

        if len(args) == 2:
            if args[0] == 'conf':
                # PPP設定削除
                intf = args[1]

                try:
                    models.vPPP.delete({'name' : intf})
                    self.update_ppp_interface(intf)

                    return {'result' : True}
                except:
                    return {'result' : False,
                            'msg'    : u"PPP '%s' の設定削除に失敗しました" % intf}

        raise cherrypy.HTTPError('400', 'PPP_dialin: Bad Request.')

    #-----------------------------------
    # Update daemon
    #-----------------------------------
    def update_daemon(self):
        # 有効 / 無効 → Daemontools
        try:
            dialinconf = models.DialIn.select_all()[0]
            dialin_enable = dialinconf.get('enable', False)
        except:
            # PPP着信設定無し
            dialin_enable = False

        if dialin_enable == True:
            self.__daemon.enable(ServiceName)
        else:
            self.__daemon.disable(ServiceName)


#-------------------------------------------------------------------------------
# Self Test
#-------------------------------------------------------------------------------
if __name__ == '__main__':
    class Empty(BaseModule.BaseModule):
        exposed = True
        def __init__(self):
            BaseModule.BaseModule.__init__(self)

        def write_config(self):
            return {'result'  : 0,
                    'message' : 'Test OK'}

    def get_password(username):
        return 'admin'

    print "## PPP Module test"
    #res = Resource.Resource()
    #conf = Config.Config()

    ppp_dialup = PPP_OCNv6(res, conf)

    # /conf/datetime
    root = Empty()
    root.conf = Empty()
    root.conf.net = Empty()
    root.conf.net.ocnv6 = ocnv6

    cp_conf = {'global' :
               {'request.show_tracebacks' : False},
               '/' :
               {'request.dispatch' : cherrypy.dispatch.MethodDispatcher(),

                'tools.wsse_auth.on'           : True,
                'tools.wsse_auth.realm'        : 'This Site',
                'tools.wsse_auth.get_password' : get_password,

                'tools.dejsonifier.on' : True,
                'tools.jsonifier.on'   : True,
                }}
    
    cherrypy.quickstart(root, '/', config=cp_conf)
