#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# MA-4xx/8xx/E2xx Web Application Server Module
#   Reboot/Shutdown module
# Author : Takeyoshi Kikuchi <kikuchi@centurysys.co.jp>
# Copyright (C) 2007-2008 Century Systems Co.,Ltd.
#
# Last Modified: 2010/05/26 14:18:36 kikuchi
#

import commands
import cherrypy
import re
import syslog
import copy
import os

# Configuration DB
from DB import config, models

from Tools import jsonlib
from Tools import wsse_auth

from Handlers import BaseModule
from Utils import Shconf
from Utils import MyTimer

CONFIG_FILE = '/etc/maconf'
REBOOT_CRONTAB = '/etc/macrontab'
REBOOT_COMMAND = '/opt/mae2xx/tools/log_reset.py -n -m reboot_by_cron'

default_conf = {
    'ROOTFS_MODE': 'RW',
    'REBOOT_ON_PPPD_FAILURE': False,
    'REBOOT_BY_CRON': False,
    'REBOOT_SCHEDULE_DAY': 'daily',
    'REBOOT_SCHEDULE_TIME': '00:00'
}

week_tab = {
    'daily': '*',
    'sunday': '0',
    'monday': '1',
    'tuesday': '2',
    'wednesday': '3',
    'thursday': '4',
    'friday': '5',
    'saturday': '6'
}

#===============================================================================
#
# Reboot/Shutdown module
#
#===============================================================================
class Reboot(BaseModule.BaseModule):
    exposed = True

    def __init__(self, debug=False, prefix='output_test/'):
        # SuperClass (BaseModule)
        self.__super = BaseModule.BaseModule

        # Superclass Constructor
        self.__super.__init__(self)

        self.log_header = "Reboot"

    def __shutdown(self):
        cmdline = "shutdown -h now"
        commands.getstatusoutput(cmdline)

    def __reboot(self):
        cmdline = "shutdown -r now"
        commands.getstatusoutput(cmdline)

    #-----------------------------------
    # HTTP method handler (GET)
    #-----------------------------------
    def GET(self, *args, **kwargs):
        obj = self.__load()

        return {
            'rootfs_mode': obj.get('ROOTFS_MODE', 
                default_conf['ROOTFS_MODE']),
            'reboot_on_pppd_failure': obj.get('REBOOT_ON_PPPD_FAILURE',
                default_conf['REBOOT_ON_PPPD_FAILURE']),
            'reboot_by_cron': obj.get('REBOOT_BY_CRON',
                default_conf['REBOOT_BY_CRON']),
            'reboot_schedule_day': obj.get('REBOOT_SCHEDULE_DAY', 
                default_conf['REBOOT_SCHEDULE_DAY']),
            'reboot_schedule_time': obj.get('REBOOT_SCHEDULE_TIME', 
                default_conf['REBOOT_SCHEDULE_TIME'])
        }

    #-----------------------------------
    # HTTP method handler (POST)
    #-----------------------------------
    def POST(self, *args, **kwargs):
        self.log(syslog.LOG_INFO, "POST: args = %s, kwargs = %s" % \
                (str(args), str(kwargs)))

        if not 'mode' in kwargs:
            raise cherrypy.HTTPError('400', "Reboot: 'mode' required.")

        mode   = kwargs['mode'].lower()

        if mode == 'reboot':
            timer = MyTimer.MyTimer(3, self.__reboot)
            msg = u"再起動しています。再起動完了後に再接続して下さい。"
        elif mode == 'shutdown':
            timer = MyTimer.MyTimer(3, self.__shutdown)
            msg = u"シャットダウンしています。完了後に電源を切ってください。"
        else:
            raise cherrypy.HTTPError('400', "Reboot: 'mode' is invalid, '%s'" % mode)
        timer.start()

        return {'result' : 'OK',
                'msg'    : msg}

    #-----------------------------------
    # HTTP method handler (PUT)
    #-----------------------------------
    def PUT(self, *args, **kwargs):
        self.log(syslog.LOG_INFO, "PUT: args = %s, kwargs = %s" % \
                (str(args), str(kwargs)))

        rootfs_mode = kwargs.get('rootfs_mode', 'RW')
        reboot_on_pppd_failure = kwargs.get('reboot_on_pppd_failure', False)
        reboot_by_cron = kwargs.get('reboot_by_cron', False)
        reboot_schedule_day = kwargs.get('reboot_schedule_day', 'daily')
        reboot_schedule_time = kwargs.get('reboot_schedule_time', '00:00')

        obj = self.__load()
        obj.update({
            'ROOTFS_MODE': rootfs_mode,
            'REBOOT_ON_PPPD_FAILURE': reboot_on_pppd_failure,
            'REBOOT_BY_CRON': reboot_by_cron,
            'REBOOT_SCHEDULE_DAY': reboot_schedule_day,
            'REBOOT_SCHEDULE_TIME': reboot_schedule_time
        })

        self.__dump(obj)
        models.ConfigDB.set('Rootfs', 'mount_mode', rootfs_mode)
        models.ConfigDB.set('Reboot', 
            'reboot_on_pppd_failure', reboot_on_pppd_failure)
        models.ConfigDB.set('Reboot', 'reboot_by_cron', reboot_by_cron)
        models.ConfigDB.set('Reboot', 
            'reboot_schedule_day', reboot_schedule_day)
        models.ConfigDB.set('Reboot', 
            'reboot_schedule_time', reboot_schedule_time)

        return {'result': True}

    #-----------------------------------
    # HTTP method handler (DELETE)
    #-----------------------------------
    def DELETE(self, *args, **kwargs):
        raise cherrypy.HTTPError('400', "Reboot: Bad request.")

    #---------------------------------------------
    # 設定ファイル書きだし
    #---------------------------------------------
    def write_data(self):
        rootfs_mode = models.ConfigDB.select_by_kind_key('Rootfs', 'mount_mode')
        if not rootfs_mode:
            rootfs_mode = default_conf['ROOTFS_MODE']

        reboot_on_pppd_failure = models.ConfigDB.select_by_kind_key(
                'Reboot', 'reboot_on_pppd_failure')
        if not reboot_on_pppd_failure:
            reboot_on_pppd_failure = False
        else:
            reboot_on_pppd_failure = True

        reboot_by_cron = models.ConfigDB.select_by_kind_key(
                'Reboot', 'reboot_by_cron')
        if not reboot_by_cron:
            reboot_by_cron = False
        else:
            reboot_by_cron = True

        reboot_schedule_day = models.ConfigDB.select_by_kind_key(
                'Reboot', 'reboot_schedule_day')
        if not reboot_schedule_day:
            reboot_schedule_day = default_conf['REBOOT_SCHEDULE_DAY']

        reboot_schedule_time = models.ConfigDB.select_by_kind_key(
                'Reboot', 'reboot_schedule_time')
        if not reboot_schedule_time:
            reboot_schedule_time = default_conf['REBOOT_SCHEDULE_TIME']

        obj = self.__load()
        obj.update({
            'ROOTFS_MODE': rootfs_mode,
            'REBOOT_ON_PPPD_FAILURE': reboot_on_pppd_failure,
            'REBOOT_BY_CRON': reboot_by_cron,
            'REBOOT_SCHEDULE_DAY': reboot_schedule_day,
            'REBOOT_SCHEDULE_TIME': reboot_schedule_time 
        })
        self.__dump(obj)

    def __load(self):
        try:
            conf = Shconf.Shconf(CONFIG_FILE)
            obj = conf.load()
        except Exception, err:
            self.log(syslog.LOG_INFO, 
                    "__load() error: '%s'" % str(err))
            obj = default_conf
        return obj

    def __dump(self, obj):
        try:
            conf = Shconf.Shconf(CONFIG_FILE)
            conf.dump(obj, header="MA-E2xx system configuration")
        except Exception, err:
            self.log(syslog.LOG_ERR,
                    "__dump() error: '%s'" % str(err))
            return False

        ret = self.__dump_crontab(obj)

        return ret

    def __dump_crontab(self, obj):
        try:
            flag = obj.get("REBOOT_BY_CRON", False)
            if flag:
                if not "REBOOT_SCHEDULE_DAY":
                    return False
                sche_day = obj["REBOOT_SCHEDULE_DAY"]
                w = week_tab.get(sche_day)
                if not w:
                    return False

                if not "REBOOT_SCHEDULE_TIME":
                    return False
                sche_time = obj["REBOOT_SCHEDULE_TIME"]
                t = sche_time.split(":")

                tab = "%s %s * * %s %s\n" % (t[1], t[0], w, REBOOT_COMMAND)

                self.log(syslog.LOG_INFO,
                        "__dump_crontab(): crontab '%s'." % tab)
                fp = open(REBOOT_CRONTAB, 'w+')
                fp.write(tab)
                fp.close()

            else:
                self.log(syslog.LOG_INFO, "__dump_crontab(): delete crontab.")
                os.unlink(REBOOT_CRONTAB)

        except Exception, err:
            self.log(syslog.LOG_ERR,
                    "__dump_crontab() error: '%s'" % str(err))

