#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# MA-4xx/8xx/E2xx Web Application Server Module
#   Static Route module
# Author : Takeyoshi Kikuchi <kikuchi@centurysys.co.jp>
# Copyright (C) 2007-2008 Century Systems Co.,Ltd.
#
# Last Modified: 2010/05/26 09:27:25 kikuchi
#

import copy
import cherrypy
import syslog

import pyiproute

# Configuration DB
from DB import config, models

from Tools import jsonlib
from Tools import wsse_auth

from Utils import RCscript
from Utils import Utils

from Handlers import BaseModule

ConfigKey = "Route"


#===============================================================================
#
# Static Route module
#
#===============================================================================
class Route(BaseModule.BaseModule):
    exposed = True

    def __init__(self, debug=False, prefix='output_test/'):
        # SuperClass (BaseModule)
        self.__super = BaseModule.BaseModule

        # モジュール内部データ管理クラス
        self.__route = InternalData(self)

        # Superclass Constructor
        self.__super.__init__(self)

        self.log_header = "Route"

    #-----------------------------------
    # HTTP method handler (GET)
    #-----------------------------------
    def GET(self, *args, **kwargs):
        return self.__route.handle_GET(*args, **kwargs)

    #-----------------------------------
    # HTTP method handler (POST)
    #-----------------------------------
    def POST(self, *args, **kwargs):
        return self.__route.handle_POST(*args, **kwargs)

    #-----------------------------------
    # HTTP method handler (PUT)
    #-----------------------------------
    def PUT(self, *args, **kwargs):
        return self.__route.handle_PUT(*args, **kwargs)

    #-----------------------------------
    # HTTP method handler (DELETE)
    #-----------------------------------
    def DELETE(self, *args, **kwargs):
        return self.__route.handle_DELETE(*args, **kwargs)

    #---------------------------------------------
    # 設定ファイル書きだし
    #---------------------------------------------
    def write_data(self):
        parent = self.get_parentmodule()
        parent.write_data("route")


#-------------------------------------------------------------------------------
# 内部データ管理
#-------------------------------------------------------------------------------
class InternalData:
    #-----------------------------------
    #
    #-----------------------------------
    def __init__(self, module):
        self.__module  = module
        self.__ipaddr  = pyiproute.ipaddr()
        self.__iproute = pyiproute.iproute()

    #=======================================================
    # GET (Retrieve)
    #=======================================================
    def handle_GET(self, *args, **kwargs):
        # **kwargs is ignored
        self.__module.log(syslog.LOG_INFO, 
                "'GET' : *args = %s, **kwargs = %s" % (str(args), str(kwargs)))

        route_info = models.Route.select_all()
        route_nums = len(route_info)

        for i in range(route_nums):
            netmask = Utils.prefixlen2netmask(route_info[i]['prefixlen'])
            route_info[i]['netmask'] = netmask

        return {'nums'   : route_nums,
                'routes' : route_info}

    #=======================================================
    # POST (Create, New)
    #=======================================================
    def handle_POST(self, *args, **kwargs):
        self.__module.log(syslog.LOG_INFO, 
            "'POST' : *args = %s, **kwargs = %s" % (str(args), str(kwargs)))
        # todo

    #=======================================================
    # PUT (Update)
    #=======================================================
    def handle_PUT(self, *args, **kwargs):
        self.__module.log(syslog.LOG_INFO, 
            "'PUT' : *args = %s, **kwargs = %s" % (str(args), str(kwargs)))

        try:
            route_info = kwargs['routes']
        except Exception, msg:
            raise cherrypy.HTTPError('400', "Route: Error '%s'" % str(msg))

        print "OK, update start..."

        deleted = models.Route.delete_all()
        print "*** StaticRoute deleted: %d" % deleted

        for info in route_info:
            prefixlen = Utils.netmask2prefixlen(str(info['netmask']))
            models.Route.insert_or_update({'address'    : info['address'],
                                           'prefixlen'  : prefixlen,
                                           'gw_address' : info['gw_address'],
                                           'gw_dev'     : info['gw_dev']})

        self.__module.write_data()
        self.__update_route()

        return {'result' : True}

    #-----
    #
    def __update_route(self):
        # delete all route
        self.__delete_all_static_routes()
        self.__add_static_routes()

    #-----
    #
    def __delete_all_static_routes(self):
        while True:
            deleted = False
            all_routes = self.__iproute.get()

            for idx in range(len(all_routes)):
                route = all_routes[idx]
                dst_addr = route.get('dst', {}).get('addr', None)

                if  dst_addr == None:
                    self.__module.log(syslog.LOG_ERR,
                        "__delete_all_static_routes: pyiproute module error!")
                    return False
                elif dst_addr == 'default':
                    print "*** skipping default route..."
                    continue
                else:
                    self.__iproute.delete(idx)
                    deleted = True
                    break

            if deleted == False:
                break

        # success
        return True

    #-----
    #
    def __add_static_routes(self):
        __devices = self.__ipaddr.get_device_list()

        devices = []
        for dev in __devices:
            if dev.startswith('eth') == True or \
                dev.startswith('ppp') == True or \
                dev.startswith('gtn') == True:
                devices.append(dev)

        routes = models.Route.select_all()

        for route in routes:
            dev = route['gw_dev']

            # 存在するデバイスのみ適用
            if dev in devices:
                try:
                    if dev.startswith('ppp') == True or \
                        dev.startswith('gtn') == True:

                        # PPP or GRE
                        scope = 'link'
                        gateway = {'dev' : dev}

                    elif dev.startswith('eth') == True:
                        # Ethernet
                        scope = 'global'
                        gateway = {'dev' : dev,
                                   'via' : route['gw_address']}
                    else:
                        self.__module.log(syslog.LOG_INFO,
                            "__add_static_routes: device '%s' currently not supported." % dev)
                        continue

                    newroute = {}
                    newroute['src']     = {}
                    newroute['family']  = 'AF_INET'
                    newroute['proto']   = route['proto']
                    newroute['dst']     = {'addr'      : route['address'],
                                           'prefixlen' : route['prefixlen']}
                    newroute['flags']   = []
                    newroute['scope']   = scope
                    newroute['table']   = 'main'
                    newroute['type']    = 'unicast'
                    newroute['gateway'] = gateway

                    self.__module.log(syslog.LOG_INFO,
                            "__add_static_routes: try to add route... %s" % str(newroute))

                    self.__iproute.add(newroute)
                except Exception, msg:
                    self.__module.log(syslog.LOG_ERR,
                            "__add_static_routes: Error, '%s'" % str(msg))


#-------------------------------------------------------------------------------
# Self Test
#-------------------------------------------------------------------------------
if __name__ == '__main__':
    class Empty(BaseModule.BaseModule):
        exposed = True
        def __init__(self):
            BaseModule.BaseModule.__init__(self)

        def write_config(self):
            return {'result'  : 0,
                    'message' : 'Test OK'}

    def get_password(username):
        return 'admin'

    print "## Static Route Module test"
    #res = Resource.Resource()
    #conf = Config.Config()

    route = Route(res, conf)

    # /conf/datetime
    root = Empty()
    root.conf = Empty()
    root.conf.net = Empty()
    root.conf.net.route = route

    cp_conf = {'global' :
               {'request.show_tracebacks' : False},
               '/' :
                {'request.dispatch' : cherrypy.dispatch.MethodDispatcher(),

                 'tools.wsse_auth.on'           : True,
                 'tools.wsse_auth.realm'        : 'This Site',
                 'tools.wsse_auth.get_password' : get_password,

                 'tools.dejsonifier.on' : True,
                 'tools.jsonifier.on'   : True,
                 }}

    cherrypy.quickstart(root, '/', config=cp_conf)
