#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# MA-E250/F pppd(callout) wrapper
# Author : Takeyoshi Kikuchi <kikuchi@centurysys.co.jp>
# Copyright (C) 2009-2012 Century Systems Co.,Ltd.
#
# Last Modified: 2013/11/26 15:55:56 kikuchi
#

import os
import sys
import re
import glob
import time
import subprocess
import commands
import signal
import syslog
import uevent
import serial
import datetime
import threading
import traceback
import pyiproute
from pyiproute import FILTER, AF

debug = False
#debug = True

supported_platforms = [
    "Century Systems MA-8xx",
    "Century Systems Magnolia2"]

FOMA_device = "/dev/ttyUM02"

local_settings = {}
localdata = {}

name = 'ppp_wrapper'
version = '1.4'

log_filename = "/var/log/foma_ppp_log"


#===============================================================================
#
#
#
#===============================================================================
class FOMAmodule(object):
    """
    FOMAユビキタスモジュール制御ライブラリ
    """
    #---------------------------------------------
    #
    #---------------------------------------------
    def __init__(self, port='/dev/ttyUSB0', baud=460800):
        """
        @param port: serial port
        @param baud: baudrate
        @type port: str
        @type baud: int
        """

        platform = self.__get_platform()
        print platform

        try:
            self.__platform_idx = supported_platforms.index(platform)
        except Exception, msg:
            errmsg = "Platform '%s' is not supported, '%s'" % (platform, str(msg))
            syslog.syslog(syslog.LOG_ERR, errmsg)
            raise ValueError

        msg = "[FOMAmodule] Platform is '%s', index = %d" % \
              (platform, self.__platform_idx)
        syslog.syslog(syslog.LOG_INFO, msg)

        if self.__platform_idx == 0:
            # MA-8xx
            self.__douts = {0 : dout(0),
                            1 : dout(1),
                            2 : dout(2),
                            3 : dout(3)} #: GPIO 'OUT' instances

            self.__din = din() #: GPIO 'IN' instance
        else:
            self.__douts = {0 : dout_magnolia2('/dev/null'),
                            1 : dout_magnolia2('/dev/null'),
                            2 : dout_magnolia2('/dev/null'),
                            3 : dout_magnolia2('/proc/driver/foma_pwrkey')}
            # MA-E2xx
            self.__pat = {'POWER' : re.compile(" +LEDG: +([01])"),
                          'SIM'   : re.compile(" +SIM_CD: +([01])")}

        self.__port = port #: SerialPort
        self.__baud = baud #: BaudRate

        self.__serial_initialized = False #: SerialPort 初期化済み flag

        # regex
        self.__commands = {
            # 電話番号
            'tel_no' : {
                'cmd' : '+CNUM',
                'idx' : [4, 1],
                'regex' : {
                    'pattern' : re.compile('\+CNUM:,"([0-9]+)",([0-9]+)'),
                    'idx'     : [0,]}
                },
            # 製造者
            'manufacturer' : {
                'cmd' : '+GMI',
                'idx' : [4, 1],
                'regex' : {
                    'pattern' : re.compile('(.+)\r\n'),
                    'idx'     : [0,]}
                },
            # 製品名
            'modelname' : {
                'cmd' : '+GMM',
                'idx' : [4, 1],
                'regex' : {
                    'pattern' : re.compile('(.+)\r\n'),
                    'idx'     : [0,]}
                },
            # 製造番号
            'serialno' : {
                'cmd' : '+CGSN',
                'idx' : [4, 1],
                'regex' : {
                    'pattern' : re.compile('(.+)\r\n'),
                    'idx'     : [0,]}
                },
            # バージョン
            'version' : {
                'cmd' : '+GMR',
                'idx' : [4, 1],
                'regex' : {
                    'pattern' : re.compile('(.+)\r\n'),
                    'idx'     : [0,]}
                },
            # アンテナレベル
            'antenna_level' : {
                'cmd' : '*DANTE',
                'idx' : [4, 1],
                'regex' : {
                    'pattern' : re.compile('\*DANTE:([0123])\r\n'),
                    'idx'     : [0,]}
                }
            }

    #---------------------------------------------
    #
    #---------------------------------------------
    def __del__(self):
        self.close()

    #---------------------------------------------
    #
    #---------------------------------------------
    def __get_platform(self):
        pat = re.compile("Hardware\t: (.+)\n")
        lines = open('/proc/cpuinfo').readlines()
        platform = "unknown"

        for l in lines:
            match = pat.match(l)
            if match:
                platform = match.groups()[0]
                break

        return platform

    #---------------------------------------------
    #
    #---------------------------------------------
    def __parse_proc(self, key):
        if not key in self.__pat:
            return 1

        pat = self.__pat[key]
        lines = open('/proc/driver/foma_status').readlines()

        stat = 1
        for line in lines:
            match = pat.match(line)
            if match:
                stat = int(match.groups()[0])
                break

        return stat

    #---------------------------------------------
    #
    #---------------------------------------------
    def get_power(self):
        """
        [API] モジュール電源 ON/OFF 状態取得

        @return: 電源状態
        @rtype: bool
        """

        if self.__platform_idx == 0:
            # MA-8xx
            if self.__douts[1].get() == False:
                return False

            # DIN: 1面に切り替え
            self.__douts[0].set(True)
            val = self.__din.get()

            return bool(val & 0x01)
        else:
            # MA-E2xx
            stat = self.__parse_proc('POWER')
            return bool(not stat)

    #---------------------------------------------
    #
    #---------------------------------------------
    def is_sim_inserted(self):
        """
        [API] SIM挿入状態取得

        @return: (SIM状態, コメント)
        @rtype: tuple
        """

        if self.get_power() == False:
            return (False, "Power is off.")

        if self.__platform_idx == 0:
            # MA-8xx

            # DIN: 1面に切り替え
            self.__douts[0].set(True)
            val = self.__din.get()

            return (bool(val & 0x08), "Power is on.")
        else:
            # MA-E2xx
            stat = self.__parse_proc('SIM')
            return (bool(not stat), "Power is on.")

    #---------------------------------------------
    #
    #---------------------------------------------
    def set_power(self, onoff):
        """
        [API] モジュール電源 ON/OFF制御

        @param onoff: 電源設定
        @type onoff: bool

        @return: 電源状態
        @rtype: bool
        """

        stat = self.get_power()

        try:
            print "* set_power: now power is '%s'" % {True  : 'On',
                                                      False : 'Off'}[stat]
        except:
            pass

        if stat == False:
            # 電源: OFF状態
            if onoff == True:
                # PWRKEY: OPEN
                self.__douts[3].set(True)

                # 電源供給: ON
                self.__douts[1].set(True)
                time.sleep(2)

                # PWRKEY: GND
                self.__douts[3].set(False)
                time.sleep(3)

                # PWRKEY: OPEN
                self.__douts[3].set(True)

                timeout = 5
                while timeout > 0:
                    tmp = self.get_power()
                    if tmp == True:
                        break
                    else:
                        timeout -= 0.2
                        time.sleep(0.2)
        else:
            # 電源: ON状態
            if onoff == False:
                # PWRKEY: OPEN -> GND
                self.__douts[3].set(True)
                time.sleep(0.1)
                self.__douts[3].set(False)
                time.sleep(1)
                # PWRKEY: OPEN
                self.__douts[3].set(True)

                timeout = 5
                while timeout > 0:
                    tmp = self.get_power()
                    if tmp == False:
                        break
                    else:
                        timeout -= 0.2
                        time.sleep(0.2)

                time.sleep(1)

                # 電源供給: OFF
                self.__douts[1].set(False)

                self.__serial_init = False

        return self.get_power()

    #---------------------------------------------
    #
    #---------------------------------------------
    def available_info(self):
        """
        [API] 取得可能全情報一覧取得

        @return: 取得用キー文字列リスト
        @rtype: list
        """

        return self.__commands.keys()

    #---------------------------------------------
    #
    #---------------------------------------------
    def init_serial(self):
        """
        [API] シリアルポート初期化

        @return: 初期化成否
        @rtype: bool
        """

        if self.get_power():
            if self.__serial_initialized == False:
                self.__s = serial.Serial(self.__port, self.__baud)

                self.__s.setTimeout(0.1)
                self.__s.write('ATE0\r\n')
                self.__s.readlines()

                self.__serial_initialized = True
        else:
            self.__serial_initialized = False

        return self.__serial_initialized

    #---------------------------------------------
    #
    #---------------------------------------------
    def get_info(self, kind):
        """
        [API] FOMAモジュール 情報取得

        @param kind: 取得キー (e.g. 'tel_no')
        @type kind: str

        @return: (結果, 取得できた情報)
        @rtype: tuple
        """

        if not self.__serial_initialized:
            return (False, "serial not initialized.")

        params = self.__commands.get(kind, None)

        if not params:
            raise KeyError

        s = self.__s

        s.write('AT%s\r\n' % params['cmd'])
        res = s.readlines()

        if len(res) != params['idx'][0]:
            return (False, None)

        pattern = params['regex']['pattern']

        match_obj = pattern.match(res[params['idx'][1]])

        if not match_obj:
            return (False, "regex not match.")

        return match_obj.groups()

    #---------------------------------------------
    #
    #---------------------------------------------
    def close(self):
        """
        [API] シリアルポート close
        """

        try:
            self.__s.close()
        except:
            pass


#===============================================================================
#
#
#
#===============================================================================
class dout(object):
    """
    GPIO 'OUTPUT' (MA-8xx)
    """
    #---------------------------------------------
    #
    #---------------------------------------------
    def __init__(self, idx):
        self.__idx = idx
        self.__f = open('/sys/class/leds/dout%d/brightness' % idx, 'w+')

    #---------------------------------------------
    #
    #---------------------------------------------
    def get(self):
        self.__f.seek(0)
        self.__f.flush()
        return bool(int(self.__f.readline()))

    #---------------------------------------------
    #
    #---------------------------------------------
    def set(self, onoff):
        brightness = int(onoff)

        #print "* dout::set idx = %d, brightness -> %d" % \
        #    (self.__idx, brightness)

        self.__f.seek(0)
        self.__f.write("%d\n" % brightness)
        self.__f.flush()


#===============================================================================
#
#
#
#===============================================================================
class dout_magnolia2(object):
    """
    GPIO 'OUTPUT' emulation (Magnolia2)
    """
    #---------------------------------------------
    #
    #---------------------------------------------
    def __init__(self, node):
        self.__f = open(node, 'w+')

    #---------------------------------------------
    #
    #---------------------------------------------
    def get(self):
        return True

    #---------------------------------------------
    #
    #---------------------------------------------
    def set(self, onoff):
        self.__f.seek(0)
        self.__f.write("%d\n" % int(onoff))
        self.__f.flush()


#===============================================================================
#
#
#
#===============================================================================
class din(object):
    """
    GPIO 'INPUT' (MA-8xx)
    """
    #---------------------------------------------
    #
    #---------------------------------------------
    def __init__(self):
        self.__f = open('/proc/driver/din', 'r')

    #---------------------------------------------
    #
    #---------------------------------------------
    def get(self):
        self.__f.seek(0)
        self.__f.flush()
        return int(self.__f.readline(), 16)


#===============================================================================
#
# PPP monitor
#
#===============================================================================
class ppp_monitor(threading.Thread):
    #---------------------------------------------
    #
    #---------------------------------------------
    def __init__(self, unit=0, demand=False):
        self.__name = "ppp_monitor"

        if demand == False:
            self.__ppp_up = False
        else:
            self.__ppp_up = None

        self.__running = False
        self.__dev_status = {}

        self.set_unit(unit)

        self.__prev_event = None

        threading.Thread.__init__(self)
        self.setDaemon(True)

    #---------------------------------------------
    #
    #---------------------------------------------
    def __mon_init(self):
        # open ipmonitor
        mon = pyiproute.ipmonitor()

        mon.open(family=AF.INET,
                 filter=[FILTER.LINK, FILTER.IPV4_IFADDR])

        return mon

    #---------------------------------------------
    #
    #---------------------------------------------
    def is_ppp_up(self):
        return self.__ppp_up

    #---------------------------------------------
    #
    #---------------------------------------------
    def unit2peeraddr(self, unit):
        return '.'.join([str(x) for x in
                         [((0xff << (3-x) * 8) & (0x0a707070 + unit)) >> (3-x) * 8 for x in xrange(4)]
                         ])

    #---------------------------------------------
    #
    #---------------------------------------------
    def set_unit(self, unit):
        self._unit = unit
        self._peeraddr = self.unit2peeraddr(unit)

        syslog.syslog("# [%s]::set_unit peer address: %s" % \
                      (self.__name, self._peeraddr))

    #---------------------------------------------
    #
    #---------------------------------------------
    def __handle_event(self, eventList):
        if not isinstance(eventList, list):
            errmsg = "! [%s]::__handle_event eventList isn't instance of 'list'" % \
                     self.__name
            syslog.syslog(syslog.LOG_ERR, errmsg)
            return

        for event in eventList:
            #print "# [%s]::__handle_event received event is ... %s" % \
            #    (self.__name, str(event))
            if event == self.__prev_event:
                msg = "# [%s]::__handle_event duplicate event." % self.__name
                syslog.syslog(syslog.LOG_INFO, msg)
                continue

            self.__prev_event = event

            try:
                kind = event[0]
                info = event[1]
            except Exception, msg:
                errmsg = "! [%s]::__handle_event event entry is invalid, '%s'" % \
                         (self.__name, str(msg))
                syslog.syslog(syslog.LOG_ERR, errmsg)
                continue

            intf = None
            new_stat = 'UNKNOWN'

            if kind == 'NEWADDR' or kind == 'DELADDR':
                # 'IFADDR' event
                intf = info['label']

                if kind == 'NEWADDR' and ('peer' in info and 'local' in info):
                    msg = "* [%s] %s: (local: %s, peer: %s)" % \
                          (self.__name, info.get('label', 'UNKNOWN'),
                           info['local'], info['peer'])
                    syslog.syslog(syslog.LOG_INFO, msg)

                    new_stat = 'UP'
                else:
                    new_stat = 'DOWN'

            old_stat = self.__dev_status.get(intf, 'DOWN')

            if intf and intf.startswith('ppp'):
                if old_stat == 'DOWN' and new_stat == 'UP':
                    # DOWN --> LOWER_UP
                    msg = "* [%s]::__handle_event PPP '%s' DOWN->UP detected." % \
                          (self.__name, intf)
                    syslog.syslog(syslog.LOG_INFO, msg)

                    if self.__ppp_up == None:
                        syslog.syslog("demand... ppp_up None -> False")
                        self.__ppp_up = False
                    else:
                        self.__ppp_up = True
                elif old_stat == 'up' and new_stat == 'DOWN':
                    # UP --> DOWN
                    msg = "* [%s]::__handle_event PPP '%s' UP->DOWN detected." % \
                          (self.__name, intf)
                    syslog.syslog(syslog.LOG_INFO, msg)

            if intf:
                self.__dev_status[intf] = new_stat

    #---------------------------------------------
    #
    #---------------------------------------------
    def run(self):
        mon = self.__mon_init()
        self.__running = True

        while self.__running == True:
            try:
                (nums, events) = mon.receive()
            except Exception, msg:
                errmsg = "! [%s]::run mon.receive() failed, '%s'" % \
                         (self.__name, str(msg))
                syslog.syslog(syslog.LOG_ERR, errmsg)

                # re-open
                try:
                    mon.close()
                    del(mon)
                except:
                    pass

                mon = self.__mon_init()
                continue

            if nums >= 1:
                try:
                    self.__handle_event(events)
                except Exception, msg:
                    errmsg = "! [%s]::run __handle_event() failed, '%s'" % \
                             (self.__name, str(msg))
                    syslog.syslog(syslog.LOG_ERR, errmsg)
            else:
                msg = "! [%s]::run received event count is 0." % \
                      self.__name
                syslog.syslog(syslog.LOG_ALERT, msg)

        try:
            mon.close()
            del(mon)
        except:
            pass

        msg = "* [%s]::run thread exit..." % self.__name
        syslog.syslog(syslog.LOG_INFO, msg)


#===============================================================================
#
# pppd watcher
#
#===============================================================================
class pppd_watcher(object):
    #---------------------------------------------
    #
    #---------------------------------------------
    def __init__(self):
        self._ppp_up = False
        self._handle = None

        self._patterns = {
            # key : (regex, terminate, ppp up/down)
            'start'     : (re.compile("Starting link"), False, -1),
            'ip_up'     : (re.compile("Script /etc/ppp/ip-up finished"), False, 1),
            'ip_down'   : (re.compile("Script /etc/ppp/ip-down started"), False, 0),
            'hangup'    : (re.compile("Modem hangup"), True, 0),
            'terminate' : (re.compile("Connection terminated"), True, 0),
            'timeout'   : (re.compile("timeout [a-zA-Z]+ Config-Requests"), True, 0)}

    #---------------------------------------------
    #
    #---------------------------------------------
    def set_handle(self, handle):
        print "self._handle = %s" % str(handle)
        self._handle = handle

    #---------------------------------------------
    #
    #---------------------------------------------
    def watch(self):
        self._ppp_up = False

        if not self._handle:
            return (False, False, False)

        terminate = False
        while True:
            try:
                f = self._handle.stdout

                if f.closed:
                    print "closed..."
                    break

                out = f.readline()

                if not out:
                    break

                out = out.rstrip()

                for (key, pat) in self._patterns.items():
                    match = pat[0].match(out)
                    if match:
                        msg = "### detect: %s" % key
                        syslog.syslog(syslog.LOG_INFO, msg)

                        if pat[2] == 1:
                            self._ppp_up = True
                        elif pat[2] == -1:
                            self._ppp_up = False

                        if pat[1] == True:
                            msg = "! terminate pppd"
                            syslog.syslog(syslog.LOG_INFO, msg)
                            terminate = True
                            break

                if terminate == True:
                    break
            except IOError, msg:
                errmsg = "# get_output failed, IOError '%s'" % msg
                print errmsg
            except Exception, msg:
                errmsg = "!!! get_output failed, %s" % msg
                print errmsg
                log_traceback()

        msg = "pppd_watcher::watch stop"
        syslog.syslog(syslog.LOG_INFO, msg)
        return (True, terminate, self._ppp_up)


#===============================================================================
#
# ppp_wrapper main
#
#===============================================================================

#---------------------------------------
#
#---------------------------------------
def make_pidfile(ppp):
    pidfile = "/var/run/ppp_wrapper.%s" % str(ppp)
    pid = os.getpid()

    fd = open(pidfile, 'w')
    fd.write("%d" % pid)
    fd.close()

    localdata['pidfile'] = pidfile

#---------------------------------------
#
#---------------------------------------
def del_pidfile():
    pidfile = localdata.get('pidfile', None)

    if pidfile != None:
        syslog.syslog("delete pidfile: %s" % pidfile)
        try:
            os.unlink(pidfile)
        except:
            pass

#---------------------------------------
#
#---------------------------------------
def device_exists(dev):
    count = len(glob.glob("/dev/%s" % dev))

    if count > 0:
        return True
    else:
        return False

#---------------------------------------
#
#---------------------------------------
def wait_for_device(dev):
    ev = uevent.uevent()

    while True:
        (result, event) = ev.receive()

        if result == True and event[0] == 'add':
            if re.match("/class/tty/(tty.+)", event[1]) == None:
                continue

            # wait for udev
            time.sleep(1)

            if device_exists(dev) == True:
                break

    return True

#---------------------------------------
#
#---------------------------------------
def make_resultfile(unit, code):
    # do nothing
    if 0:
        resultfile = "/var/state/ppp/result.ppp%d" % unit
        fd = open(resultfile, 'w')
        fd.write("%d" % code)
        fd.close()

#---------------------------------------
#
#---------------------------------------
def sig_handler(signum, frame):
    syslog.syslog("sig_handler: signal no. %s" % str(signum))

    if 'child_pppd' in localdata:
        child_pppd = localdata['child_pppd']

        try:
            os.kill(child_pppd.pid, signal.SIGTERM)
        except Exception, msg:
            errmsg = "sig_handler: os.kill() failed, '%s'" % \
                     str(msg)
            syslog.syslog(syslog.LOG_WARNING, errmsg)

        localdata['sig_rcvd'] = True
        localdata['signum'] = signum
        exitcode = child_pppd.wait()
        syslog.syslog("sig_handler: exitcode = %d" % exitcode)

        localdata['exitcode'] = exitcode
    else:
        del_pidfile()
        sys.exit(0)

#---------------------------------------
#
#---------------------------------------
def alarm_handler(signum, frame):
    msg = "# alarm_handler: signal no. %s" % str(signum)
    syslog.syslog(syslog.LOG_INFO, msg)

    ipaddr = pyiproute.ipaddr()

    try:
        peername = local_settings['peername']

        ipaddr.get()
        pppx_info = ipaddr.get_IPv4address(peername)
        syslog.syslog(syslog.LOG_INFO, " %s is '%s'" % (peername, str(pppx_info)))
    except Exception, msg:
        errmsg = "! alarm_handler: caught exception, '%s'" % str(msg)
        syslog.syslog(syslog.LOG_ERR, errmsg)

        pppx_info = None

    if pppx_info and pppx_info[0].has_key('local'):
        msg = "# alarm_handler: %s is connected." % peername
        syslog.syslog(syslog.LOG_INFO, msg)
        signal.alarm(0)

        return
    else:
        msg = "! alarm_handler: %s is not connected." % peername
        log_to_file(msg)
        syslog.syslog(syslog.LOG_ERR, msg)

    localdata['sig_rcvd'] = True
    localdata['signum'] = signum
    localdata['exitcode'] = 0

    raise IOError, "SIGALRM"

#---------------------------------------
#
#---------------------------------------
def parse_chat_result():
    try:
        lines = open('/tmp/chat_result', 'r').readlines()
    except Exception, msg:
        syslog.syslog("open(chat_result_file) failed, '%s'" % str(msg))
        return (0, None)

    # ex.
    # exitcode: 7
    # ABORT: 'NO CARRIER'

    pat_exitcode = re.compile("exitcode: ([0-9]+)")
    pat_string   = re.compile("ABORT: '([^']+)'")

    err = [0, 'UNKNOWN']

    for line in lines:
        match = pat_exitcode.match(line)
        if match:
            err[0] = int(match.groups()[0])
            continue

        match = pat_string.match(line)
        if match:
            err[1] = match.groups()[0]

    if not err[1]:
        err[1] = 'UNKNOWN'

    return tuple(err)

#---------------------------------------
#
#---------------------------------------
def reset_modem(dev, speed):
    s = serial.Serial(dev, speed)

    time.sleep(2)

    s.setTimeout(1)
    s.write('AT\r\n')
    s.write('AT*DHWRST\r\n')

    while True:
        l = s.readline()
        if not l:
            break
        syslog.syslog("modem: %s" % l)

    syslog.syslog("modem reset completed.")
    s.close()

#---------------------------------------
#
#---------------------------------------
def hangup_modem(dev, speed):
    s = serial.Serial(dev, speed)

    time.sleep(2)

    s.setTimeout(1)
    s.write('ATH\r\n')

    while True:
        l = s.readline()
        if not l:
            break
        try:
            syslog.syslog(syslog.LOG_ERR, "modem: length %d" % len(l))
            syslog.syslog(syslog.LOG_ERR, "modem: %s" % str(l))
        except Exception, msg:
            pass

    syslog.syslog("modem hangup completed.")
    s.close()

#---------------------------------------
#
#---------------------------------------
def power_off_on_modem(count):
    # power off/on
    foma = FOMAmodule(FOMA_device)

    msg = "FOMA module power off/on (count: %d) start..." % count
    log_to_file(msg)
    syslog.syslog(msg)

    msg = "FOMA module -> power off"
    log_to_file(msg)
    syslog.syslog(msg)

    res = foma.set_power(False)

    if res == True:
        # power off fail
        msg = "power off failed, reboot..."
        log_to_file(msg)
        syslog.syslog(msg)

        commands.getoutput("/sbin/shutdown -r now")
        time.sleep(20)
    else:
        msg = "power off succeeded."
        log_to_file(msg)
        syslog.syslog(msg)

    time.sleep(20)
    msg = "FOMA module -> power on"
    log_to_file(msg)
    syslog.syslog(msg)

    res = foma.set_power(True)

    if res == False:
        # power on fail
        msg = "power on failed, reboot..."
        log_to_file(msg)
        syslog.syslog(msg)

        commands.getoutput("/sbin/shutdown -r now")
        time.sleep(20)
    else:
        msg = "power on succeeded."
        log_to_file(msg)
        syslog.syslog(msg)

#---------------------------------------
#
#---------------------------------------
def sync():
    commands.getoutput("sync")

#---------------------------------------
#
#---------------------------------------
def force_reset():
    for x in xrange(3):
        sync()

    commands.getoutput("/bin/busybox reboot -f")

#---------------------------------------
#
#---------------------------------------
def log_to_file(text):
    try:
        now = datetime.datetime.now().isoformat(' ').split('.')[0]
        open(log_filename, 'a+').write("%s: %s\n" % (now, text))

        sync()
    except:
        pass

#---------------------------------------
#
#---------------------------------------
def parse_peers(peername):
    try:
        peerfile = "/etc/ppp/peers/%s" % peername
        fp = open(peerfile, 'r')
    except Exception, msg:
        # 発生しないはず
        errmsg = "parse_peers: %s not found." % peerfile
        syslog.syslog(syslog.LOG_ERR, errmsg)
        return None

    flags = []
    _dev = None
    unit = 0
    noipdefault = False

    for line in fp:
        key = line.strip()

        if key in ['persist', 'nopersist', 'demand']:
            flags.append(key)
        elif key == 'noipdefault':
            noipdefault = True

        # unit
        match = re.match("^unit ([\d]+)$", line)
        if match:
            unit = int(match.groups()[0])

        if not _dev:
            try:
                _dev = re.match("^/dev/(tty.+)$", line).groups()[0]
            except AttributeError:
                try:
                    _dev = re.match("^plugin rp-pppoe.so (eth[01])$", line).groups()[0]
                except AttributeError:
                    pass

    persist = False
    demand = False

    if 'demand' in flags:
        demand = True

        if 'nopersist' in flags:
            persist = False
        else:
            persist = True
    elif 'persist' in flags:
        persist = True

    print "parse_peers: dev=%s, persist=%s, demand=%s, unit=%d, noipdefault=%s" % \
        (_dev, str(persist), str(demand), unit, str(noipdefault))
    return (_dev, persist, demand, unit, noipdefault)

#---------------------------------------
#
#---------------------------------------
def kill_process(child):
    try:
        kill_retry = 10

        while kill_retry > 0:
            try:
                os.kill(child.pid, signal.SIGKILL)
                time.sleep(2)
            except:
                pass

            poll = child.poll()

            if poll:
                break
            else:
                msg = "! pppd not terminated, retry to KILL (%d)..." % \
                      kill_retry
                syslog.syslog(syslog.LOG_INFO, msg)
                time.sleep(5)
                kill_retry -= 1

        if poll:
            syslog.syslog("# pppd returncode is %d" % poll)
            child.wait()
        else:
            errmsg = "! kill pppd failed (???), force reset!"
            syslog.syslog(syslog.LOG_ERR, errmsg)
            try:
                force_reset()
            except:
                pass

            sys.exit(1)
    except Exception, msg:
        errmsg = "! caught exception, '%s'" % str(msg)
        syslog.syslog(syslog.LOG_ERR, errmsg)

#---------------------------------------
#
#---------------------------------------
def log_traceback():
    try:
        err_traceback = traceback.format_exc()
        print err_traceback
        log.logger(log.LOG_ERR, err_traceback)
    except:
        pass




#---------------------------------------
#
#---------------------------------------
def connect_ppp(cmdline):
    # PPPmonitor
    #pppmon = ppp_monitor(local_settings['pppunit'],
    #                     local_settings['demand'])
    #pppmon.start()

    err = [0, '']

    for retry in xrange(3):
        localdata.clear()
        sig_rcvd = False

        if local_settings['demand'] == False:
            # set SIGALRM handler (timeout 180 sec)
            signal.signal(signal.SIGALRM, alarm_handler)
            signal.alarm(180)

        child = subprocess.Popen(cmdline, close_fds=True,
                                 stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        localdata['child_pppd'] = child

        pppwatch = pppd_watcher()
        pppwatch.set_handle(child)

        terminate = False
        try:
            (res, terminate, is_ppp_up) = pppwatch.watch()
            if terminate == True:
                syslog.syslog("==> terminate pppd...")
                kill_process(child)

            pppd_rc = child.wait()
        except OSError, msg:
            # SIGALRM, restart
            msg = "# Caught SIGNAL, restarting child.wait()..."
            syslog.syslog(syslog.LOG_INFO, msg)

            try:
                (res, terminate, is_ppp_up) = pppwatch.watch()
                if terminate == True:
                    syslog.syslog("==> terminate pppd...")
                    kill_process(child)

                pppd_rc = child.wait()
            except Exception, msg:
                pppd_rc = localdata.get('exitcode', 0)
        except IOError, msg:
            try:
                errmsg = "! pppd timeouted... child pid is %d" % child.pid
            except:
                errmsg = "! pppd timeouted..."

            syslog.syslog(syslog.LOG_ERR, errmsg)
            kill_process(child)

            pppd_rc = -9
        except Exception, msg:
            print msg
            log_traceback()
            pppd_rc = localdata.get('exitcode', 0)

        syslog.syslog("pppd finished (exitcode: %d, retry: %d)" % \
                      (pppd_rc, retry))

        # restore SIGALRM handler
        signal.signal(signal.SIGALRM, orig_handler['SIGALRM'])
        signal.alarm(0)

        try:
            hangup_modem("/dev/%s" % dev, 460800)
        except Exception, msg:
            errmsg = "hangup_modem caught exception, '%s'" % msg
            syslog.syslog(syslog.LOG_ERR, errmsg)
            pass

        if not 'sig_rcvd' in localdata:
            make_resultfile(unit, 4)

        #is_ppp_up = pppmon.is_ppp_up()
        msg = "is_ppp_up while pppd alive -> %s" % str(is_ppp_up)
        syslog.syslog(syslog.LOG_INFO, msg)

        if pppd_rc == 8:
            # chat script failed
            err = parse_chat_result()

            syslog.syslog("err = %s" % str(err))

            if err[1].startswith("*DANTE"):
                # out of range
                syslog.syslog("Out of Range")

                if retry < 2:
                    time.sleep(10)
                    continue
            elif err[1].startswith('NO CARRIER'):
                syslog.syslog("NO CARRIER")

                if retry < 2:
                    time.sleep(10)
                    continue
            elif err[0] == 3:
                syslog.syslog("chat timeouted...")

                if retry < 2:
                    time.sleep(10)
                    continue
        elif pppd_rc == -9 or pppd_rc == 16:
            if terminate == True:
                # modem hangup
                if is_ppp_up == False:
                    # LCP片方向 (modem bug)
                    err[0] = 256
            elif is_ppp_up == False:
                syslog.syslog(syslog.LOG_ERR, "pppd connection timeouted, restart...")

                if retry < 2:
                    time.sleep(30)
                    continue
            else:
                # killed by ip-down
                pppd_rc = 0

        break

    return (pppd_rc, err)


#===========================================================
# main
#===========================================================
if __name__ == '__main__':
    syslog.openlog(name)

    syslog.syslog("ppp_wrapper version %s" % version)

    try:
        ppp = sys.argv[1]
    except:
        print "usage: python %s ppp[0-9]+" % sys.argv[0]
        sys.exit(1)

    try:
        unit = int(re.match("ppp([0-9]+)", ppp).groups()[0])
    except AttributeError:
        sys.exit(2)

    syslog.openlog("ppp_wrapper")
    local_settings['unit'] = unit
    local_settings['peername'] = peername = "ppp%d" % unit

    # find device, persist
    (dev, _persist, _demand, pppunit, noipdefault) = parse_peers(peername)

    local_settings['persist'] = _persist
    local_settings['demand']  = _demand
    local_settings['pppunit'] = pppunit
    local_settings['noipdefault'] = noipdefault

    if dev == None:
        print "'/dev/tty?' or 'eth[01]' not found."
        make_resultfile(unit, 3)
        sys.exit(3)

    syslog.syslog("device is '/dev/%s'" % dev)

    # make pidfile
    try:
        make_pidfile("ppp%d" % unit)
    except Exception, msg:
        errmsg = "! make_pidfile() caught exception, '%s'" % str(msg)
        syslog.syslog(syslog.LOG_ERR, errmsg)
        sys.exit(3)

    orig_handler = {}
    orig_handler['SIGTERM'] = signal.getsignal(signal.SIGTERM)
    orig_handler['SIGINT']  = signal.getsignal(signal.SIGINT)
    orig_handler['SIGALRM'] = signal.getsignal(signal.SIGALRM)

    signal.signal(signal.SIGTERM, sig_handler)
    signal.signal(signal.SIGINT, sig_handler)

    if dev.startswith('tty') and not device_exists(dev):
        wait_for_device(dev)

    syslog.syslog("pppd start (unit=%d)" % unit)

    # device ok, exec pppd (nodetach)
    cmdline = ["/usr/sbin/pppd", "call", "ppp%d" % unit, "nodetach"]

    power_off_on_count = 0
    while True:
        for retry in xrange(3):
            try:
                (pppd_rc, err) = connect_ppp(cmdline)

                # test
                #tmp = list(err)
                #tmp[0] = 256
                #err = tuple(tmp)
            except Exception, msg:
                errmsg = "! connect_ppp() caught exception, '%s'" % str(msg)
                syslog.syslog(syslog.LOG_ERR, errmsg)
                log_traceback()
                continue
                #sys.exit(1)

            msg = "connect_ppp() -> rc=%d, err=%s" % (pppd_rc, str(err))
            syslog.syslog(syslog.LOG_INFO, msg)

            need_powerctrl = False
            continue_retry = False

            if err[0] == 3:
                syslog.syslog("chat timeouted, try modem power OFF/ON...")

                need_powerctrl = True
            elif pppd_rc == 12:
                syslog.syslog("Disconnected by idle timeout")
            elif pppd_rc == 16 or err[0] == 256:
                errmsg = "One-way IPCP/LCP (modem and/or FOMA bug) (count: %d)" % (retry+1)
                log_to_file(errmsg)
                syslog.syslog(syslog.LOG_ERR, errmsg)

                reset_modem("/dev/%s" % dev, 460800)
                time.sleep(10)

                need_powerctrl = True
                continue_retry = True

            if err[1] == "NO CARRIER" or err[1].startswith("*DANTE"):
                syslog.syslog("resetting modem")
                reset_modem("/dev/%s" % dev, 460800)
                time.sleep(10)

                need_powerctrl = True
                continue_retry = True

            if continue_retry and retry < 2:
                syslog.syslog("waiting 10 seconds...")
                time.sleep(10)
            else:
                # pppd normaly exit
                break

        # persist/nopersit
        if pppd_rc == 5 or not _persist:
            break

    del_pidfile()

    # restore signal handlers
    signal.signal(signal.SIGTERM, orig_handler['SIGTERM'])
    signal.signal(signal.SIGINT, orig_handler['SIGINT'])

    syslog.syslog("ppp_wrapper finished...")
    sys.exit(pppd_rc)
