#! /bin/bash
# MA-8xx/E2xx Firmware update script
#
# Copyright 2007-2009 Century Systems Co.,Ltd.
# Author : Takeyoshi Kikuchi <kikuchi@centurysys.co.jp>
#
# Last Modified: 2009/05/29 12:49:23 kikuchi
#

# Gentoo Linux common functions
source /etc/init.d/functions.sh

#===========================================================
# Global definitions
#===========================================================
OPERATION_MODE="PRODUCTION"
#OPERATION_MODE="DEBUG"

VERSION='1.00'

# Firmware Filename
#FIRMWARE_FILE='/tmp/MA8xxFirm.img'
FIRMWARE_FILE='/tmp/MAE2xxFirm.img'

# Size
SIZE_FIRMWARE=33030144

SIZE_ROOTFS=29360128
SIZE_KERNEL=3670016
SIZE_ROOT_KERNEL=$((SIZE_ROOTFS+SIZE_KERNEL))

# for dd (kernel)
DD_BLOCKSIZE=65536
DD_COUNT=56
DD_OFFSET=448
MTDBLOCK_KERNEL='/dev/mtdblock3'

# for flashcp (rootfs)
MTD_ROOTFS='/dev/mtd4'

# Firmup Flags
FIRMUP_FLAG='/tmp/FIRM.UP'

# Watchdog Driver
WATCHDOG_MOD='softdog'
WATCHDOG_PARAMS='soft_margin=1'
WATCHDOG_DEV='/tmp/watchdog'
WATCHDOG_INSTALLED="NO"
MXC_WDT="NO"

#=====================================================================
#
# Utility functions
#
#=====================================================================

#-------------------------------------------------
# char * tolower(char *text)
#
#-------------------------------------------------
function tolower {
	echo $1 | tr '[A-Z]' '[a-z]'
}

#-----------------------------------------------------------
# void print_msg(int result, char *msg)
#
# if result != 0 --> print msg
#-----------------------------------------------------------
function print_msg {
	if [ "${result}" != "0" ]; then
		echo ${msg}
	fi
}

#-----------------------------------------------------------
# int truncate_file(char *file, int newsize)
#
# success --> 0
# fail    --> 1
#-----------------------------------------------------------
function truncate_file {
	local filename=$1
	local newsize=$2
	local result
	local msg

	ebegin "Truncate file ${filename} to ${newsize} bytes"

	if [ "${newsize}" == "" ] || [ "${filename}" == "" ]; then
		result=1
		msg=" !!! filename \& newsize is required."
	else
		truncate -s ${newsize} ${filename} &> /dev/null
		result=$?

		if [ "${result}" != "0" ]; then
			msg="truncate_file failed."
		fi
	fi

	eend ${result}
	print_msg "${result}" "${msg}"

	return ${result}
}

#-----------------------------------------------------------
# int check_operation_mode(void)
#
# production --> 0
# debug      --> 1
#-----------------------------------------------------------
function check_operation_mode {
	local result

	if [ "${OPERATION_MODE}" == "PRODUCTION" ]; then
		result=0
	else
		result=1
	fi

	return ${result}
}

#-----------------------------------------------------------
# void __exit reboot(void)
#
#-----------------------------------------------------------
function reboot {
	if [ "${WATCHDOG_INSTALLED}" != "YES" ]; then
		exit 1
	fi

	if [ "${MXC_WDT}" = "NO" ]; then
		# Reset! (start watchdog timer)
		echo R > ${WATCHDOG_DEV}
		sleep 5
	fi

	# not reached here
	exit 0
}

#-----------------------------------------------------------
# void ledctrl(char *led, char *mode, ...)
#
# firmup required     --> 0
# firmup not required --> 1
#-----------------------------------------------------------
function ledctrl {
	local leddir='/sys/class/leds/led_'$1
	local mode=$2

	if [ "${mode}" == "on" ]; then
		echo 'default-on' > ${leddir}/trigger
		echo 1 > ${leddir}/brightness
	elif [ "${mode}" == "off" ]; then
		echo 'default-on' > ${leddir}/trigger
		echo 0 > ${leddir}/brightness
	elif [ "${mode}" == "blink" ]; then
		local ontime=$3
		local offtime=$4

		echo 'timer' > ${leddir}/trigger
		echo $ontime > ${leddir}/delay_on
		echo $offtime > ${leddir}/delay_off
	fi
}

#=====================================================================
#
# Firmware check functions
#
#=====================================================================

#-----------------------------------------------------------
# int check_firmware_size(char *file)
#
# ok         --> 0
# not found  --> 1
# size error --> 2
#-----------------------------------------------------------
function check_firmware_size {
	local filename=$1
	local size_stat
	local result
	local msg

	ebegin "Checking Firmware ${filename}"
	if [ ! -f "${filename}" ]; then
		msg="File not found."
		result=1
	else
		# Busybox 'stat'
		size_stat=`stat -c "%s" ${filename}`

		if [ "${size_stat}" != "${SIZE_FIRMWARE}" ]; then
			result=2
			msg="Filesize mismatch ${size_stat}"
		else
			result=0
		fi
	fi

	eend ${result}
	print_msg "${result}" "${msg}"

	return ${result}
}

#-----------------------------------------------------------
# int is_firmup_required(void)
#
# firmup required     --> 0
# firmup not required --> 1
#-----------------------------------------------------------
function is_firmup_required {
	local result

	if [ -f "${FIRMUP_FLAG}" ]; then
		result=0
	else
		result=1
	fi

	return ${result}
}

#=====================================================================
#
# Firmware update functions
#
#=====================================================================

#-----------------------------------------------------------
# void update_kernel(void)
#
#-----------------------------------------------------------
function update_kernel {
	local cmdline
	local result

	echo "*** Updating Kernel ***"

	# Resize firmware to (rootfs + kernel) size
	truncate_file ${FIRMWARE_FILE} ${SIZE_ROOT_KERNEL}
	result=$?

	if [ "${result}" != "0" ]; then
		exit 1
	fi

	# LED No.0 : Red 200ms, OFF 200ms blink
	#echo "BLINK:0 R:200,K:200" > ${LEDCTLD_FIFO}
	ledctrl g0 off
	ledctrl r0 blink 200 200

	# Writing Kernel Image to mtd area
	cmdline="dd if=${FIRMWARE_FILE} of=${MTDBLOCK_KERNEL} bs=${DD_BLOCKSIZE} skip=${DD_OFFSET} count=${DD_COUNT}"

	if check_operation_mode; then
		ebegin "Writing Kernel Image to ${MTDBLOCK_KERNEL}"
		eval "${cmdline} &> /dev/null"
		result=$?
		eend ${result}

		if [ "${result}" != "0" ]; then
			echo "   !!! update_kernel() failed!"
			exit 1
		fi
	else
		echo "   debug: cmdline = \"${cmdline}\""
	fi

	# LED No.0 : OFF
	#echo "SET:0 K" > ${LEDCTLD_FIFO}
	ledctrl r0 off
}

#-----------------------------------------------------------
# void update_rootfs(void)
#
#-----------------------------------------------------------
function update_rootfs {
	local cmdline
	local result

	echo "*** Updating rootfs ***"

	# Resize firmware to rootfs size
	truncate_file ${FIRMWARE_FILE} ${SIZE_ROOTFS}
	result=$?

	if [ "${result}" != "0" ]; then
		exit 1
	fi

	# LED No.0 : Red 500ms, Green 500ms blink
	#echo "BLINK:0 R:500,G:500" > ${LEDCTLD_FIFO}
	ledctrl g0 blink 500 500
	sleep 0.5
	ledctrl r0 blink 500 500

	# Writing rootfs Image
	cmdline="flashcp -v ${FIRMWARE_FILE} ${MTD_ROOTFS}"

	if check_operation_mode; then
		echo "   Writing rootfs Image to ${MTD_ROOTFS}"
		eval ${cmdline}
		#echo $cmdline

		echo "   Reboot !"
		reboot
	else
		echo "   debug: cmdline = \"${cmdline}\""
		echo "   Reboot !"
	fi
}

#===============================================================================
#
# Main
#
#===============================================================================

echo "*** MA-E2xx Firmware updater version ${VERSION}"

# ファームウェアのファイルサイズをチェック
check_firmware_size "${FIRMWARE_FILE}"
result=$?

if [ "${result}" != "0" ]; then
	exit 0
fi

# ファームアップが必要かチェック
is_firmup_required
result=$?

if [ "${result}" != "0" ]; then
	echo "   FirmupFlag '${FIRMUP_FLAG}' not found."
	exit 0
else
	echo "   FirmupFlag found."
fi


lsmod | grep mxc_wdt &> /dev/null
if [ "$?" = "0" ]; then
	ebegin "Launch mxc_wdt watchdog clearing script"
	/opt/sbin/clear_wdt.sh &
	result=$?
	MXC_WDT="YES"
else
	# Watchdog Driver を組み込めるかチェック
	ebegin "Try to insmod Watchdog driver"
	modprobe ${WATCHDOG_MOD} ${WATCHDOG_PARAMS} &> /dev/null
	result=$?

	if [ "${result}" == "0" ]; then
		WATCHDOG_INSTALLED="YES"
		mknod /tmp/watchdog c 10 130
	fi
fi

eend ${result}

if [ "${result}" != 0 ]; then
	exit 1
fi

# アップデート (Kernel)
update_kernel

# アップデート (rootfs)
update_rootfs

# not reached here
exit 0
